import json
import logging

from pathlib import Path
from enum import (
    Enum,
    IntEnum,
)

LOGGER = logging.getLogger(__package__)


class Features(Enum):
    AUX_FAN = 1,
    CHAMBER_LIGHT = 2,
    CHAMBER_FAN = 3,
    CHAMBER_TEMPERATURE = 4,
    CURRENT_STAGE = 5,
    PRINT_LAYERS = 6,
    AMS = 7,
    EXTERNAL_SPOOL = 8,
    K_VALUE = 9,
    START_TIME = 10,
    AMS_TEMPERATURE = 11,
    CAMERA_RTSP = 13,
    START_TIME_GENERATED = 14,
    CAMERA_IMAGE = 15,
    DOOR_SENSOR = 16,
    MANUAL_MODE = 17,
    AMS_FILAMENT_REMAINING = 18,


class FansEnum(Enum):
    PART_COOLING = 1,
    AUXILIARY = 2,
    CHAMBER = 3,
    HEATBREAK = 4,


CURRENT_STAGE_IDS = {
    "default": "unknown",
    0: "printing",
    1: "auto_bed_leveling",
    2: "heatbed_preheating",
    3: "sweeping_xy_mech_mode",
    4: "changing_filament",
    5: "m400_pause",
    6: "paused_filament_runout",
    7: "heating_hotend",
    8: "calibrating_extrusion",
    9: "scanning_bed_surface",
    10: "inspecting_first_layer",
    11: "identifying_build_plate_type",
    12: "calibrating_micro_lidar", # DUPLICATED?
    13: "homing_toolhead",
    14: "cleaning_nozzle_tip",
    15: "checking_extruder_temperature",
    16: "paused_user",
    17: "paused_front_cover_falling",
    18: "calibrating_micro_lidar", # DUPLICATED?
    19: "calibrating_extrusion_flow",
    20: "paused_nozzle_temperature_malfunction",
    21: "paused_heat_bed_temperature_malfunction",
    22: "filament_unloading",
    23: "paused_skipped_step",
    24: "filament_loading",
    25: "calibrating_motor_noise",
    26: "paused_ams_lost",
    27: "paused_low_fan_speed_heat_break",
    28: "paused_chamber_temperature_control_error",
    29: "cooling_chamber",
    30: "paused_user_gcode",
    31: "motor_noise_showoff",
    32: "paused_nozzle_filament_covered_detected",
    33: "paused_cutter_error",
    34: "paused_first_layer_error",
    35: "paused_nozzle_clog",
    # X1 returns -1 for idle
    -1: "idle",  # DUPLICATED
    # P1 returns 255 for idle
    255: "idle", # DUPLICATED
}

CURRENT_STAGE_OPTIONS = list(set(CURRENT_STAGE_IDS.values())) # Conversion to set first removes the duplicates

GCODE_STATE_OPTIONS = [
    "failed",
    "finish",
    "idle",
    "init",
    "offline",
    "pause",
    "prepare",
    "running",
    "slicing",
    "unknown"
]

SPEED_PROFILE = {
    1: "silent",
    2: "standard",
    3: "sport",
    4: "ludicrous"
}

PRINT_TYPE_OPTIONS = {
    "cloud",
    "local",
    "idle",
    "system",
    "unknown"
}


def load_dict(filename: str) -> dict:
    with open(filename) as f:
        return json.load(f);


FILAMENT_NAMES = load_dict(Path(__file__).with_name('filaments.json'))

# TODO: Update error lists with data from https://e.bambulab.com/query.php?lang=en
# UNIQUE_ID=dAa5VFRi
HMS_ERRORS = {
    "0C00_0300_0002_0010": "Foreign objects detected on hotbed: please check and clean the hotbed.",
    "0500_0100_0003_0004": "Not enough space in MicroSD Card; please clear some space.",
    "0300_0200_0001_0006": "The nozzle temperature is abnormal; the sensor may have a short circuit. Please check whether the connector is properly plugged in.",
    "0500_0300_0001_0021": "Hardware incompatible: please check the laser.",
    "0500_0500_0001_0001": "The factory data of AP board is abnormal; please replace the AP board with a new one.",
    "0500_0500_0003_0002": "The device is in the engineering state; please pay attention to the information security related matters.",
    "0300_1800_0001_0005": "The Z axis motor got stuck while moving, or the extrusion force sensor may have an issue; please check if there is any foreign matter on the Z sliders or Z timing belt wheels.",
    "0300_0200_0001_000B": "The nozzle temperature is abnormal. Temperature control system may have an issue.",
    "0300_9100_0001_000C": "The chamber heater 1 has been working at full load for a long time. The temperature control system may have an issue.",
    "0300_0100_0001_000D": "An issue occurred when heating the heatbed previously. To continue using your printer, please refer to the wiki to troubleshoot.",
    "0500_0300_0001_0024": "The current temperature is too low. In order to protect you and your printer. Printing task, moving axis and other operations is disabled. Please move the printer to an environment above 10°C.",
    "0500_0200_0002_0008": "Time synchronization failed.",
    "0300_1A00_0002_0002": "The nozzle is clogged with filament.",
    "0300_0F00_0001_0001": "Abnormal accelerometer data detected. Please try to restart the printer.",
    "0300_0100_0001_0008": "An abnormality occurs during the heating process of the heatbed, the heating modules may be broken.",
    "0300_0100_0001_000C": "The heatbed has worked at full load for a long time. The temperature control system may be abnormal.",
    "0500_0200_0002_0006": "Streaming function error. Please check the network and try again. You can restart or update the printer if the issue persists.",
    "0300_0D00_0002_0004": "The build plate is not placed properly. Please adjust it.",
    "0300_0D00_0002_0005": "The build plate is not placed properly. Please adjust it.",
    "0300_0D00_0002_0006": "The build plate is not placed properly. Please adjust it.",
    "0300_0D00_0002_0007": "The build plate is not placed properly. Please adjust it.",
    "0300_0D00_0002_0008": "The build plate is not placed properly. Please adjust it.",
    "0300_0D00_0002_0009": "The build plate is not placed properly. Please adjust it.",
    "0300_0D00_0002_000A": "The build plate is not placed properly. Please adjust it.",
    "0300_0D00_0002_0003": "The build plate is not placed properly. Please adjust it.",
    "1201_1000_0001_0001": "The AMS2 Slot1 motor has slipped. The extrusion wheel may be malfunctioning, or the filament may be too thin.",
    "1202_1000_0001_0001": "The AMS3 Slot1 motor has slipped. The extrusion wheel may be malfunctioning, or the filament may be too thin.",
    "1201_1000_0002_0002": "The AMS2 Slot1 motor is overloaded. The filament may be tangled or stuck.",
    "1202_1000_0002_0002": "The AMS3 Slot1 motor is overloaded. The filament may be tangled or stuck.",
    "1203_1000_0002_0002": "The AMS4 Slot1 motor is overloaded. The filament may be tangled or stuck.",
    "1201_1100_0002_0002": "The AMS2 Slot2 motor is overloaded. The filament may be tangled or stuck.",
    "1202_1100_0002_0002": "The AMS3 Slot2 motor is overloaded. The filament may be tangled or stuck.",
    "1203_1100_0002_0002": "The AMS4 Slot2 motor is overloaded. The filament may be tangled or stuck.",
    "1201_1200_0002_0002": "The AMS2 Slot3 motor is overloaded. The filament may be tangled or stuck.",
    "1202_1200_0002_0002": "The AMS3 Slot3 motor is overloaded. The filament may be tangled or stuck.",
    "1203_1200_0002_0002": "The AMS4 Slot3 motor is overloaded. The filament may be tangled or stuck.",
    "1201_1300_0002_0002": "The AMS2 Slot4 motor is overloaded. The filament may be tangled or stuck.",
    "1202_1300_0002_0002": "The AMS3 Slot4 motor is overloaded. The filament may be tangled or stuck.",
    "1203_1300_0002_0002": "The AMS4 Slot4 motor is overloaded. The filament may be tangled or stuck.",
    "1201_1000_0001_0003": "The AMS2 Slot1 motor torque control is malfunctioning. The current sensor may be faulty.",
    "1202_1000_0001_0003": "The AMS3 Slot1 motor torque control is malfunctioning. The current sensor may be faulty.",
    "1203_1000_0001_0003": "The AMS4 Slot1 motor torque control is malfunctioning. The current sensor may be faulty.",
    "1200_1300_0002_0002": "The AMS1 Slot4 motor is overloaded. The filament may be tangled or stuck.",
    "1200_4500_0002_0003": "The filament cutter handle has not released. The handle or blade may be stuck.",
    "0C00_0300_0002_000F": "Parts skipped before first layer inspection; the inspection will not be supported for the current print.",
    "0C00_0100_0002_0008": "Failed to get image from chamber camera. Spaghetti and waste chute pileup detection is not available for now.",
    "0C00_0300_0003_0006": "Purged filament may have piled up in the waste chute. Please check and clean the chute.",
    "1200_1000_0001_0001": "The AMS1 Slot1 motor has slipped. The extrusion wheel may be malfunctioning, or the filament may be too thin.",
    "1203_1000_0001_0001": "The AMS4 Slot1 motor has slipped. The extrusion wheel may be malfunctioning, or the filament may be too thin.",
    "1200_1000_0002_0002": "The AMS1 Slot1 motor is overloaded. The filament may be tangled or stuck.",
    "1200_1100_0002_0002": "The AMS1 Slot2 motor is overloaded. The filament may be tangled or stuck.",
    "1200_1200_0002_0002": "The AMS1 Slot3 motor is overloaded. The filament may be tangled or stuck.",
    "1200_1000_0001_0003": "The AMS1 Slot1 motor torque control is malfunctioning. The current sensor may be faulty.",
    "0300_1D00_0001_0001": "The position sensor of extrusion motor is abnormal. The sensor signal wire may be not properly plugged in.",
    "0300_1C00_0001_0001": "The extrusion motor driver is abnormal. The MOSFET may have a short circuit.",
    "0300_0100_0001_000A": "The hotbed temperature control is abnormal, the AC board may be broken.",
    "0300_9100_0001_000A": "The temperature of chamber heater 1 is abnormal. The AC board may be broken.",
    "0300_9200_0001_000A": "The temperature of chamber heater 2 is abnormal. The AC board may be broken.",
    "0300_1700_0001_0001": "The speed of the nozzle fan is too slow or stopped. It may be stuck or the connector may not be plugged in properly.",
    "0300_1700_0002_0002": "The speed of the nozzle fan is slow. It may be stuck and need cleaning.",
    "0300_1800_0001_0002": "The sensitivity of the extrusion force sensor is low, the nozzle may not installed correctly.",
    "0500_0400_0003_0009": "The bed temperature exceeds filament's vitrification temperature, which may cause nozzle clog. Please keep the front door of printer open. Already turning off the door opening detection temporarily.",
    "0300_1B00_0001_0001": "The signal of the heatbed acceleration sensor is weak. The sensor may have fallen off or been damaged.",
    "0300_1B00_0001_0002": "External disturbance was detected on the heatbed acceleeration sensor. The sensor signal wire may not be fixed.",
    "0300_1B00_0001_0003": "The heatbed acceleration sensor detected unexpected continuous force. The sensor may be stuck, or the analog front end may be broken.",
    "0300_0D00_0001_000B": "The Z axis motor seems to be stuck when moving. Please check if there is any foreign matter on the Z sliders or Z timing belt wheels .",
    "0300_0D00_0002_0001": "Heatbed homing abnormal: there may be a bulge on the heatbed or the nozzle tip may not be clean.",
    "0300_1000_0002_0002": "The resonance frequency of the X axis differs greatly from last calibration. Please clean the carbon rod and rerun the machine calibration afterward.",
    "0300_1100_0002_0002": "The resonance frequency of the Y axis differs greatly from the last calibration. Please clean the carbon rod and rerun the machine calibration afterward.",
    "12FF_2000_0002_0007": "Failed to check the filament location in the tool head; please click for more help.",
    "1200_2000_0002_0006": "Failed to extrude AMS1 Slot1 filament; the extruder may be clogged or the filament may be too thin, causing the extruder to slip.",
    "1201_2000_0002_0006": "Failed to extrude AMS2 Slot1 filament; the extruder may be clogged or the filament may be too thin, causing the extruder to slip.",
    "1202_2000_0002_0006": "Failed to extrude AMS3 Slot1 filament; the extruder may be clogged or the filament may be too thin, causing the extruder to slip.",
    "1203_2000_0002_0006": "Failed to extrude AMS4 Slot1 filament; the extruder may be clogged or the filament may be too thin, causing the extruder to slip.",
    "1200_2100_0002_0006": "Failed to extrude AMS1 Slot2 filament; the extruder may be clogged or the filament may be too thin, causing the extruder to slip.",
    "1201_2100_0002_0006": "Failed to extrude AMS2 Slot2 filament; the extruder may be clogged or the filament may be too thin, causing the extruder to slip.",
    "1202_2100_0002_0006": "Failed to extrude AMS3 Slot2 filament; the extruder may be clogged or the filament may be too thin, causing the extruder to slip.",
    "1203_2100_0002_0006": "Failed to extrude AMS4 Slot2 filament; the extruder may be clogged or the filament may be too thin, causing the extruder to slip.",
    "1200_2200_0002_0006": "Failed to extrude AMS1 Slot3 filament; the extruder may be clogged or the filament may be too thin, causing the extruder to slip.",
    "1201_2200_0002_0006": "Failed to extrude AMS2 Slot3 filament; the extruder may be clogged or the filament may be too thin, causing the extruder to slip.",
    "1202_2200_0002_0006": "Failed to extrude AMS3 Slot3 filament; the extruder may be clogged or the filament may be too thin, causing the extruder to slip.",
    "1203_2200_0002_0006": "Failed to extrude AMS4 Slot3 filament; the extruder may be clogged or the filament may be too thin, causing the extruder to slip.",
    "1200_2300_0002_0006": "Failed to extrude AMS1 Slot4 filament; the extruder may be clogged or the filament may be too thin, causing the extruder to slip.",
    "1201_2300_0002_0006": "Failed to extrude AMS2 Slot4 filament; the extruder may be clogged or the filament may be too thin, causing the extruder to slip.",
    "1202_2300_0002_0006": "Failed to extrude AMS3 Slot4 filament; the extruder may be clogged or the filament may be too thin, causing the extruder to slip.",
    "1203_2300_0002_0006": "Failed to extrude AMS4 Slot4 filament; the extruder may be clogged or the filament may be too thin, causing the extruder to slip.",
    "0300_0200_0001_0009": "The nozzle temperature control is abnormal; the hot end may not be installed. If you want to heat the hot end without it being installed, please turn on maintenance mode on the settings page.",
    "0300_9400_0002_0003": "Chamber failed to reach the desired temperature. The machine will stop waiting for the chamber temperature.",
    "0500_0400_0002_0020": "",
    "0300_9000_0001_0002": "Chamber heating failed. The chamber may not be enclosed, or the ambient temperature may be too low, or the heat dissipation vent of the power supply may be blocked.",
    "0500_0300_0001_0004": "The AHB module is malfunctioning. Please restart the device.",
    "0500_0300_0002_0020": "Micro SD Card capacity is insufficient to cache print files.",
    "12FF_2000_0003_0007": "Checking the filament location of all AMS slots, please wait.",
    "0300_9400_0003_0001": "Chamber cooling may be too slow. You can open the chamber to help cooling if the gas in chamber is non-toxic.",
    "0300_1800_0001_0001": "The value of extrusion force sensor is low, the nozzle seems to not be installed.",
    "0300_1800_0001_0003": "The extrusion force sensor is not available, the link between the MC and TH may be broken or the sensor is broken.",
    "0300_1800_0001_0004": "The data from extrusion force sensor is abnormal, the sensor should be broken.",
    "0300_1900_0001_0001": "The eddy current sensor on Y-axis is not available, the wire should be broken.",
    "0300_1900_0002_0002": "The sensitivity of Y-axis eddy current sensor is too low.",
    "0300_9400_0003_0002": "Chamber temperature setting value exceed the limit, the boundary value will be set.",
    "0300_1A00_0002_0001": "The nozzle is covered with filaments, or the build plate is put in crooked.",
    "0300_9000_0001_0010": "The communication of chamber temperature controller is abnormal.",
    "0300_9100_0001_0008": "Chamber heater 1 failed to rise to target temperature.",
    "0300_9300_0001_0006": "Chamber temperature is abnormal. The chamber heater's temperature sensor at the air inlet may have an open circuit.",
    "0300_9300_0001_0007": "Chamber temperature is abnormal. The temperature sensor at the power supply may have a short circuit.",
    "0300_9300_0001_0008": "Chamber temperature is abnormal. The temperature sensor at power supply may have an open circuit.",
    "0300_9300_0001_0002": "Chamber temperature is abnormal. The chamber heater's temperature sensor may have an open circuit.",
    "0300_9300_0001_0003": "Chamber temperature is abnormal. The chamber heater's temperature sensor at the air outlet may have a short circuit.",
    "0300_9300_0001_0004": "Chamber temperature is abnormal. The chamber heater's temperature sensor at the air outlet may have an open circuit.",
    "0300_9300_0001_0005": "Chamber temperature is abnormal. The chamber heater's temperature sensor at the air inlet may have a short circuit.",
    "0300_9100_0001_0001": "The temperature of chamber heater 1 is abnormal. The heater may have a short circuit.",
    "0300_9100_0001_0002": "The temperature of chamber heater 1 is abnormal. The heater may have an open circuit or the thermal fuse may have taken effect.",
    "0300_9100_0001_0006": "The temperature of chamber heater 1 is abnormal. The sensor may have a short circuit.",
    "0300_9100_0001_0007": "The temperature of chamber heater 1 is abnormal. The sensor may have an open circuit.",
    "0300_9200_0001_0001": "The temperature of chamber heater 2 is abnormal. The heater may have a short circuit.",
    "0300_9200_0001_0002": "The temperature of chamber heater 2 is abnormal. The heater may have an open circuit or the thermal fuse may be in effect.",
    "0300_9200_0001_0006": "The temperature of chamber heater 2 is abnormal. The sensor may have a short circuit.",
    "0300_9200_0001_0007": "The temperature of chamber heater 2 is abnormal. The sensor may have an open circuit.",
    "0300_9200_0001_0008": "Chamber heater 2 failed to rise to target temperature.",
    "0300_9300_0001_0001": "Chamber temperature is abnormal. The chamber heater's temperature sensor may have a short circuit.",
    "0300_9000_0001_0001": "Chamber heating failed. The chamber heater may be failing to blow hot air.",
    "0300_9000_0001_0003": "Chamber heating failed. The temperature of power supply may be too high.",
    "0300_9000_0001_0004": "Chamber heating failed. The speed of the heating fan is too low.",
    "0300_9000_0001_0005": "Chamber heating failed. The thermal resistance is too high.",
    "0300_9100_0001_0003": "The temperature of chamber heater 1 is abnormal. The heater is over temperature.",
    "0300_9200_0001_0003": "The temperature of chamber heater 2 is abnormal. The heater is over temperature.",
    "0500_0300_0001_0023": "The CTC module is malfunctioning. Please restart the device.",
    "1200_8000_0002_0001": "AMS1 Slot1 filament may be tangled or stuck.",
    "1200_8100_0002_0001": "AMS1 Slot2 filament may be tangled or stuck.",
    "1200_8200_0002_0001": "AMS1 Slot3 filament may be tangled or stuck.",
    "1200_8300_0002_0001": "AMS1 Slot4 filament may be tangled or stuck.",
    "1201_8000_0002_0001": "AMS2 Slot1 filament may be tangled or stuck.",
    "1201_8100_0002_0001": "AMS2 Slot2 filament may be tangled or stuck.",
    "1201_8200_0002_0001": "AMS2 Slot3 filament may be tangled or stuck.",
    "1201_8300_0002_0001": "AMS2 Slot4 filament may be tangled or stuck.",
    "1202_8000_0002_0001": "AMS3 Slot1 filament may be tangled or stuck.",
    "1202_8100_0002_0001": "AMS3 Slot2 filament may be tangled or stuck.",
    "1202_8200_0002_0001": "AMS3 Slot3 filament may be tangled or stuck.",
    "1202_8300_0002_0001": "AMS3 Slot4 filament may be tangled or stuck.",
    "1203_8000_0002_0001": "AMS4 Slot1 filament may be tangled or stuck.",
    "1203_8100_0002_0001": "AMS4 Slot2 filament may be tangled or stuck.",
    "1203_8200_0002_0001": "AMS4 Slot3 filament may be tangled or stuck.",
    "1203_8300_0002_0001": "AMS4 Slot4 filament may be tangled or stuck.",
    "12FF_8000_0002_0001": "The filament on the spool holder may be tangled or stuck.",
    "0500_0400_0002_0014": "The RFID-tag on AMS2 Slot1 cannot be identified.",
    "0500_0400_0002_0013": "The RFID-tag on AMS1 Slot4 cannot be identified.",
    "0500_0400_0002_0010": "The RFID-tag on AMS1 Slot1 cannot be identified.",
    "0500_0400_0002_0011": "The RFID-tag on AMS1 Slot2 cannot be identified.",
    "0500_0400_0002_0012": "The RFID-tag on AMS1 Slot3 cannot be identified.",
    "0500_0400_0002_0015": "The RFID-tag on AMS2 Slot2 cannot be identified.",
    "0500_0400_0002_0016": "The RFID-tag on AMS2 Slot3 cannot be identified.",
    "0500_0400_0002_0017": "The RFID-tag on AMS2 Slot4 cannot be identified.",
    "0500_0400_0002_0018": "The RFID-tag on AMS3 Slot1 cannot be identified.",
    "0500_0400_0002_0019": "The RFID-tag on AMS3 Slot2 cannot be identified.",
    "0500_0400_0002_001A": "The RFID-tag on AMS3 Slot3 cannot be identified.",
    "0500_0400_0002_001B": "The RFID-tag on AMS3 Slot4 cannot be identified.",
    "0500_0400_0002_001C": "The RFID-tag on AMS4 Slot1 cannot be identified.",
    "0500_0400_0002_001D": "The RFID-tag on AMS4 Slot2 cannot be identified.",
    "0500_0400_0002_001E": "The RFID-tag on AMS4 Slot3 cannot be identified.",
    "0500_0400_0002_001F": "The RFID-tag on AMS4 Slot4 cannot be identified.",
    "0500_0300_0003_0022": "MicroSD Card performance degradation has been detected. It may affect print jobs, logs, and video records. Please try to format or change the MicroSD card.",
    "0300_0300_0002_0002": "",
    "1203_1100_0001_0001": "The AMS4 Slot2 motor has slipped. The extrusion wheel may be malfunctioning, or the filament may be too thin.",
    "1203_1200_0001_0001": "The AMS4 Slot3 motor has slipped. The extrusion wheel may be malfunctioning, or the filament may be too thin.",
    "1203_1300_0001_0001": "The AMS4 Slot4 motor has slipped. The extrusion wheel may be malfunctioning, or the filament may be too thin.",
    "1203_1100_0001_0003": "The AMS4 Slot2 motor torque control is malfunctioning. The current sensor may be faulty.",
    "1203_1200_0001_0003": "The AMS4 Slot3 motor torque control is malfunctioning. The current sensor may be faulty.",
    "1203_1300_0001_0003": "The AMS4 Slot4 motor torque control is malfunctioning. The current sensor may be faulty.",
    "1201_1100_0001_0001": "The AMS2 Slot2 motor has slipped. The extrusion wheel may be malfunctioning, or the filament may be too thin.",
    "1202_1100_0001_0001": "The AMS3 Slot2 motor has slipped. The extrusion wheel may be malfunctioning, or the filament may be too thin.",
    "1201_1200_0001_0001": "The AMS2 Slot3 motor has slipped. The extrusion wheel may be malfunctioning, or the filament may be too thin.",
    "1202_1200_0001_0001": "The AMS3 Slot3 motor has slipped. The extrusion wheel may be malfunctioning, or the filament may be too thin.",
    "1201_1300_0001_0001": "The AMS2 Slot4 motor has slipped. The extrusion wheel may be malfunctioning, or the filament may be too thin.",
    "1202_1300_0001_0001": "The AMS3 Slot4 motor has slipped. The extrusion wheel may be malfunctioning, or the filament may be too thin.",
    "1201_1100_0001_0003": "The AMS2 Slot2 motor torque control is malfunctioning. The current sensor may be faulty.",
    "1202_1100_0001_0003": "The AMS3 Slot2 motor torque control is malfunctioning. The current sensor may be faulty.",
    "1201_1200_0001_0003": "The AMS2 Slot3 motor torque control is malfunctioning. The current sensor may be faulty.",
    "1202_1200_0001_0003": "The AMS3 Slot3 motor torque control is malfunctioning. The current sensor may be faulty.",
    "1201_1300_0001_0003": "The AMS2 Slot4 motor torque control is malfunctioning. The current sensor may be faulty.",
    "1202_1300_0001_0003": "The AMS3 Slot4 motor torque control is malfunctioning. The current sensor may be faulty.",
    "12FF_2000_0002_0002": "Filament on the spool holder is empty; please insert a new filament.",
    "1200_3000_0001_0001": "AMS1 Slot1 RFID coil is broken or the RF hardware circuit has an error.",
    "1201_3000_0001_0001": "AMS2 Slot1 RFID coil is broken or the RF hardware circuit has an error.",
    "1202_3000_0001_0001": "AMS3 Slot1 RFID coil is broken or the RF hardware circuit has an error.",
    "1203_3000_0001_0001": "AMS4 Slot1 RFID coil is broken or the RF hardware circuit has an error.",
    "1200_3100_0001_0001": "AMS1 Slot2 RFID coil is broken or the RF hardware circuit has an error.",
    "1201_3100_0001_0001": "AMS2 Slot2 RFID coil is broken or the RF hardware circuit has an error.",
    "1202_3100_0001_0001": "AMS3 Slot2 RFID coil is broken or the RF hardware circuit has an error.",
    "1203_3100_0001_0001": "AMS4 Slot2 RFID coil is broken or the RF hardware circuit has an error.",
    "1200_3200_0001_0001": "AMS1 Slot3 RFID coil is broken or the RF hardware circuit has an error.",
    "1201_3200_0001_0001": "AMS2 Slot3 RFID coil is broken or the RF hardware circuit has an error.",
    "1202_3200_0001_0001": "AMS3 Slot3 RFID coil is broken or the RF hardware circuit has an error.",
    "1203_3200_0001_0001": "AMS4 Slot3 RFID coil is broken or the RF hardware circuit has an error.",
    "1200_3300_0001_0001": "AMS1 Slot4 RFID coil is broken or the RF hardware circuit has an error.",
    "1201_3300_0001_0001": "AMS2 Slot4 RFID coil is broken or the RF hardware circuit has an error.",
    "1202_3300_0001_0001": "AMS3 Slot4 RFID coil is broken or the RF hardware circuit has an error.",
    "1203_3300_0001_0001": "AMS4 Slot4 RFID coil is broken or the RF hardware circuit has an error.",
    "1200_3000_0001_0004": "RFID cannot be read because of an encryption chip failure in AMS1.",
    "1201_3000_0001_0004": "RFID cannot be read because of an encryption chip failure in AMS2.",
    "1202_3000_0001_0004": "RFID cannot be read because of an encryption chip failure in AMS3.",
    "1203_3000_0001_0004": "RFID cannot be read because of an encryption chip failure in AMS4.",
    "1200_4500_0002_0001": "The filament cutter sensor is malfunctioning. Please check whether the connector is properly plugged in.",
    "1200_5000_0002_0001": "AMS1 communication is abnormal; please check the connection cable.",
    "1201_5000_0002_0001": "AMS2 communication is abnormal; please check the connection cable.",
    "1202_5000_0002_0001": "AMS3 communication is abnormal; please check the connection cable.",
    "1203_5000_0002_0001": "AMS4 communication is abnormal; please check the connection cable.",
    "1200_5100_0003_0001": "AMS is disabled; please load filament from spool holder.",
    "1201_2000_0002_0001": "AMS2 Slot1 filament has run out; please insert a new filament.",
    "1202_2000_0002_0001": "AMS3 Slot1 filament has run out; please insert a new filament.",
    "1203_2000_0002_0001": "AMS4 Slot1 filament has run out; please insert a new filament.",
    "1200_2100_0002_0001": "AMS1 Slot2 filament has run out; please insert a new filament.",
    "1201_2100_0002_0001": "AMS2 Slot2 filament has run out; please insert a new filament.",
    "1202_2100_0002_0001": "AMS3 Slot2 filament has run out; please insert a new filament.",
    "1203_2100_0002_0001": "AMS4 Slot2 filament has run out; please insert a new filament.",
    "1200_2200_0002_0001": "AMS1 Slot3 filament has run out; please insert a new filament.",
    "1201_2200_0002_0001": "AMS2 Slot3 filament has run out; please insert a new filament.",
    "1202_2200_0002_0001": "AMS3 Slot3 filament has run out; please insert a new filament.",
    "1203_2200_0002_0001": "AMS4 Slot3 filament has run out; please insert a new filament.",
    "1200_2300_0002_0001": "AMS1 Slot4 filament has run out; please insert a new filament.",
    "1201_2300_0002_0001": "AMS2 Slot4 filament has run out; please insert a new filament.",
    "1202_2300_0002_0001": "AMS3 Slot4 filament has run out; please insert a new filament.",
    "1203_2300_0002_0001": "AMS4 Slot4 filament has run out; please insert a new filament.",
    "12FF_2000_0002_0001": "Filament at the spool holder has run out; please insert a new filament.",
    "1201_2000_0002_0002": "AMS2 Slot1 is empty; please insert a new filament.",
    "1202_2000_0002_0002": "AMS3 Slot1 is empty; please insert a new filament.",
    "1203_2000_0002_0002": "AMS4 Slot1 is empty; please insert a new filament.",
    "1200_2100_0002_0002": "AMS1 Slot2 is empty; please insert a new filament.",
    "1201_2100_0002_0002": "AMS2 Slot2 is empty; please insert a new filament.",
    "1202_2100_0002_0002": "AMS3 Slot2 is empty; please insert a new filament.",
    "1203_2100_0002_0002": "AMS4 Slot2 is empty; please insert a new filament.",
    "1200_2200_0002_0002": "AMS1 Slot3 is empty; please insert a new filament.",
    "1201_2200_0002_0002": "AMS2 Slot3 is empty; please insert a new filament.",
    "1202_2200_0002_0002": "AMS3 Slot3 is empty; please insert a new filament.",
    "1203_2200_0002_0002": "AMS4 Slot3 is empty; please insert a new filament.",
    "1200_2300_0002_0002": "AMS1 Slot4 is empty; please insert a new filament.",
    "1201_2300_0002_0002": "AMS2 Slot4 is empty; please insert a new filament.",
    "1202_2300_0002_0002": "AMS3 Slot4 is empty; please insert a new filament.",
    "1203_2300_0002_0002": "AMS4 Slot4 is empty; please insert a new filament.",
    "1203_2100_0002_0005": "AMS4 Slot2 filament has run out, and purging the old filament went abnormally; please check to see if filament is stuck in the toolhead.",
    "1201_2000_0003_0001": "AMS2 Slot1 filament has run out. Purging the old filament; please wait.",
    "1202_2000_0003_0001": "AMS3 Slot1 filament has run out. Purging the old filament; please wait.",
    "1203_2000_0003_0001": "AMS4 Slot1 filament has run out. Purging the old filament; please wait.",
    "1201_2100_0003_0001": "AMS2 Slot2 filament has run out. Purging the old filament; please wait.",
    "1202_2100_0003_0001": "AMS3 Slot2 filament has run out. Purging the old filament; please wait.",
    "1203_2100_0003_0001": "AMS4 Slot2 filament has run out. Purging the old filament; please wait.",
    "1200_2200_0003_0001": "AMS1 Slot3 filament has run out. Purging the old filament; please wait.",
    "1201_2200_0003_0001": "AMS2 Slot3 filament has run out. Purging the old filament; please wait.",
    "1202_2200_0003_0001": "AMS3 Slot3 filament has run out. Purging the old filament; please wait.",
    "1203_2200_0003_0001": "AMS4 Slot3 filament has run out. Purging the old filament; please wait.",
    "1200_2300_0003_0001": "AMS1 Slot4 filament has run out. Purging the old filament; please wait.",
    "1201_2300_0003_0001": "AMS2 Slot4 filament has run out. Purging the old filament; please wait.",
    "1202_2300_0003_0001": "AMS3 Slot4 filament has run out. Purging the old filament; please wait.",
    "1203_2300_0003_0001": "AMS4 Slot4 filament has run out. Purging the old filament; please wait.",
    "1200_2000_0003_0001": "AMS1 Slot1 filament has run out. Purging the old filament; please wait.",
    "1200_2100_0003_0001": "AMS1 Slot2 filament has run out. Purging the old filament; please wait.",
    "1200_2300_0003_0002": "AMS1 Slot4 filament has run out and automatically switched to the slot with the same filament.",
    "1201_2300_0003_0002": "AMS2 Slot4 filament has run out and automatically switched to the slot with the same filament.",
    "1202_2300_0003_0002": "AMS3 Slot4 filament has run out and automatically switched to the slot with the same filament.",
    "1203_2300_0003_0002": "AMS4 Slot4 filament has run out and automatically switched to the slot with the same filament.",
    "1201_2000_0003_0002": "AMS2 Slot1 filament has run out and automatically switched to the slot with the same filament.",
    "1202_2000_0003_0002": "AMS3 Slot1 filament has run out and automatically switched to the slot with the same filament.",
    "1203_2000_0003_0002": "AMS4 Slot1 filament has run out and automatically switched to the slot with the same filament.",
    "1200_2100_0003_0002": "AMS1 Slot2 filament has run out and automatically switched to the slot with the same filament.",
    "1201_2100_0003_0002": "AMS2 Slot2 filament has run out and automatically switched to the slot with the same filament.",
    "1202_2100_0003_0002": "AMS3 Slot2 filament has run out and automatically switched to the slot with the same filament.",
    "1203_2100_0003_0002": "AMS4 Slot2 filament has run out and automatically switched to the slot with the same filament.",
    "1200_2200_0003_0002": "AMS1 Slot3 filament has run out and automatically switched to the slot with the same filament.",
    "1201_2200_0003_0002": "AMS2 Slot3 filament has run out and automatically switched to the slot with the same filament.",
    "1202_2200_0003_0002": "AMS3 Slot3 filament has run out and automatically switched to the slot with the same filament.",
    "1203_2200_0003_0002": "AMS4 Slot3 filament has run out and automatically switched to the slot with the same filament.",
    "1202_2200_0002_0003": "AMS3 Slot3 filament may be broken in the PTFE tube.",
    "1203_2200_0002_0003": "AMS4 Slot3 filament may be broken in the PTFE tube.",
    "1200_2300_0002_0003": "AMS1 Slot4 filament may be broken in the PTFE tube.",
    "1201_2300_0002_0003": "AMS2 Slot4 filament may be broken in the PTFE tube.",
    "1202_2300_0002_0003": "AMS3 Slot4 filament may be broken in the PTFE tube.",
    "1203_2300_0002_0003": "AMS4 Slot4 filament may be broken in the PTFE tube.",
    "1200_2000_0002_0004": "AMS1 Slot1 filament may be broken in the tool head.",
    "1201_2000_0002_0004": "AMS2 Slot1 filament may be broken in the tool head.",
    "1202_2000_0002_0004": "AMS3 Slot1 filament may be broken in the tool head.",
    "1203_2000_0002_0004": "AMS4 Slot1 filament may be broken in the tool head.",
    "1200_2100_0002_0004": "AMS1 Slot2 filament may be broken in the tool head.",
    "1201_2100_0002_0004": "AMS2 Slot2 filament may be broken in the tool head.",
    "1202_2100_0002_0004": "AMS3 Slot2 filament may be broken in the tool head.",
    "1203_2100_0002_0004": "AMS4 Slot2 filament may be broken in the tool head.",
    "1200_2200_0002_0004": "AMS1 Slot3 filament may be broken in the tool head.",
    "1201_2200_0002_0004": "AMS2 Slot3 filament may be broken in the tool head.",
    "1202_2200_0002_0004": "AMS3 Slot3 filament may be broken in the tool head.",
    "1203_2200_0002_0004": "AMS4 Slot3 filament may be broken in the tool head.",
    "1200_2300_0002_0004": "AMS1 Slot4 filament may be broken in the tool head.",
    "1201_2300_0002_0004": "AMS2 Slot4 filament may be broken in the tool head.",
    "1202_2300_0002_0004": "AMS3 Slot4 filament may be broken in the tool head.",
    "1203_2300_0002_0004": "AMS4 Slot4 filament may be broken in the tool head.",
    "1200_2000_0003_0002": "AMS1 Slot1 filament has run out and automatically switched to the slot with the same filament.",
    "1200_1100_0001_0001": "The AMS1 Slot2 motor has slipped. The extrusion wheel may be malfunctioning, or the filament may be too thin.",
    "1200_1200_0001_0001": "The AMS1 Slot3 motor has slipped. The extrusion wheel may be malfunctioning, or the filament may be too thin.",
    "1200_1300_0001_0001": "The AMS1 Slot4 motor has slipped. The extrusion wheel may be malfunctioning, or the filament may be too thin.",
    "1200_1100_0001_0003": "The AMS1 Slot2 motor torque control is malfunctioning. The current sensor may be faulty.",
    "1200_1200_0001_0003": "The AMS1 Slot3 motor torque control is malfunctioning. The current sensor may be faulty.",
    "1200_1300_0001_0003": "The AMS1 Slot4 motor torque control is malfunctioning. The current sensor may be faulty.",
    "1200_2000_0002_0003": "AMS1 Slot1 filament may be broken in the PTFE tube.",
    "1201_2000_0002_0003": "AMS2 Slot1 filament may be broken in the PTFE tube.",
    "1202_2000_0002_0003": "AMS3 Slot1 filament may be broken in the PTFE tube.",
    "1203_2000_0002_0003": "AMS4 Slot1 filament may be broken in the PTFE tube.",
    "1200_2100_0002_0003": "AMS1 Slot2 filament may be broken in the PTFE tube.",
    "1201_2100_0002_0003": "AMS2 Slot2 filament may be broken in the PTFE tube.",
    "1202_2100_0002_0003": "AMS3 Slot2 filament may be broken in the PTFE tube.",
    "1203_2100_0002_0003": "AMS4 Slot2 filament may be broken in the PTFE tube.",
    "1200_2200_0002_0003": "AMS1 Slot3 filament may be broken in the PTFE tube.",
    "1201_2200_0002_0003": "AMS2 Slot3 filament may be broken in the PTFE tube.",
    "1202_2200_0002_0005": "AMS3 Slot3 filament has run out, and purging the old filament went abnormally; please check to see if filament is stuck in the toolhead.",
    "1203_2200_0002_0005": "AMS4 Slot3 filament has run out, and purging the old filament went abnormally; please check to see if filament is stuck in the toolhead.",
    "1200_2300_0002_0005": "AMS1 Slot4 filament has run out, and purging the old filament went abnormally; please check to see if filament is stuck in the toolhead.",
    "1201_2300_0002_0005": "AMS2 Slot4 filament has run out, and purging the old filament went abnormally; please check to see if filament is stuck in the toolhead.",
    "1202_2300_0002_0005": "AMS3 Slot4 filament has run out, and purging the old filament went abnormally; please check to see if filament is stuck in the toolhead.",
    "1203_2300_0002_0005": "AMS4 Slot4 filament has run out, and purging the old filament went abnormally; please check to see if filament is stuck in the toolhead.",
    "1200_2000_0002_0001": "AMS1 Slot1 filament has run out; please insert a new filament.",
    "1200_2000_0002_0002": "AMS1 Slot1 is empty; please insert a new filament.",
    "1200_2000_0002_0005": "AMS1 Slot1 filament has run out, and purging the old filament went abnormally; please check to see if filament is stuck in the toolhead.",
    "1201_2000_0002_0005": "AMS2 Slot1 filament has run out, and purging the old filament went abnormally; please check to see if filament is stuck in the toolhead.",
    "1202_2000_0002_0005": "AMS3 Slot1 filament has run out, and purging the old filament went abnormally; please check to see if filament is stuck in the toolhead.",
    "1203_2000_0002_0005": "AMS4 Slot1 filament has run out, and purging the old filament went abnormally; please check to see if filament is stuck in the toolhead.",
    "1200_2100_0002_0005": "AMS1 Slot2 filament has run out, and purging the old filament went abnormally; please check to see if filament is stuck in the toolhead.",
    "1201_2100_0002_0005": "AMS2 Slot2 filament has run out, and purging the old filament went abnormally; please check to see if filament is stuck in the toolhead.",
    "1202_2100_0002_0005": "AMS3 Slot2 filament has run out, and purging the old filament went abnormally; please check to see if filament is stuck in the toolhead.",
    "1200_2200_0002_0005": "AMS1 Slot3 filament has run out, and purging the old filament went abnormally; please check to see if filament is stuck in the toolhead.",
    "1201_2200_0002_0005": "AMS2 Slot3 filament has run out, and purging the old filament went abnormally; please check to see if filament is stuck in the toolhead.",
    "12FF_2000_0002_0005": "Filament may be broken in the tool head.",
    "12FF_2000_0002_0006": "Failed to extrude the filament and the extruder may be clogged.",
    "1200_3000_0002_0002": "The RFID-tag on AMS1 Slot1 is damaged.",
    "1201_3000_0002_0002": "The RFID-tag on AMS2 Slot1 is damaged.",
    "1202_3000_0002_0002": "The RFID-tag on AMS3 Slot1 is damaged.",
    "1203_3000_0002_0002": "The RFID-tag on AMS4 Slot1 is damaged.",
    "1200_3100_0002_0002": "The RFID-tag on AMS1 Slot2 is damaged.",
    "1201_3100_0002_0002": "The RFID-tag on AMS2 Slot2 is damaged.",
    "1202_3100_0002_0002": "The RFID-tag on AMS3 Slot2 is damaged.",
    "1203_3100_0002_0002": "The RFID-tag on AMS4 Slot2 is damaged.",
    "1200_3200_0002_0002": "The RFID-tag on AMS1 Slot3 is damaged.",
    "1201_3200_0002_0002": "The RFID-tag on AMS2 Slot3 is damaged.",
    "1202_3200_0002_0002": "The RFID-tag on AMS3 Slot3 is damaged.",
    "1203_3200_0002_0002": "The RFID-tag on AMS4 Slot3 is damaged.",
    "1200_3300_0002_0002": "The RFID-tag on AMS1 Slot4 is damaged.",
    "1201_3300_0002_0002": "The RFID-tag on AMS2 Slot4 is damaged.",
    "1202_3300_0002_0002": "The RFID-tag on AMS3 Slot4 is damaged.",
    "1203_3300_0002_0002": "The RFID-tag on AMS4 Slot4 is damaged.",
    "1200_3000_0003_0003": "AMS1 Slot1 RFID cannot be read because of a structural error.",
    "1201_3000_0003_0003": "AMS2 Slot1 RFID cannot be read because of a structural error.",
    "1202_3000_0003_0003": "AMS3 Slot1 RFID cannot be read because of a structural error.",
    "1203_3000_0003_0003": "AMS4 Slot1 RFID cannot be read because of a structural error.",
    "1200_3100_0003_0003": "AMS1 Slot2 RFID cannot be read because of a structural error.",
    "1201_3100_0003_0003": "AMS2 Slot2 RFID cannot be read because of a structural error.",
    "1202_3100_0003_0003": "AMS3 Slot2 RFID cannot be read because of a structural error.",
    "1203_3100_0003_0003": "AMS4 Slot2 RFID cannot be read because of a structural error.",
    "1200_3200_0003_0003": "AMS1 Slot3 RFID cannot be read because of a structural error.",
    "1201_3200_0003_0003": "AMS2 Slot3 RFID cannot be read because of a structural error.",
    "1202_3200_0003_0003": "AMS3 Slot3 RFID cannot be read because of a structural error.",
    "1203_3200_0003_0003": "AMS4 Slot3 RFID cannot be read because of a structural error.",
    "1200_3300_0003_0003": "AMS1 Slot4 RFID cannot be read because of a structural error.",
    "1201_3300_0003_0003": "AMS2 Slot4 RFID cannot be read because of a structural error.",
    "1202_3300_0003_0003": "AMS3 Slot4 RFID cannot be read because of a structural error.",
    "1203_3300_0003_0003": "AMS4 Slot4 RFID cannot be read because of a structural error.",
    "1200_4500_0002_0002": "The filament cutter's cutting distance is too large. The X motor may lose steps.",
    "1200_7000_0001_0001": "AMS1 Filament speed and length error: The slot 1 filament odometry may be faulty.",
    "1201_7000_0001_0001": "AMS2 Filament speed and length error: The slot 1 filament odometry may be faulty.",
    "1202_7000_0001_0001": "AMS3 Filament speed and length error: The slot 1 filament odometry may be faulty.",
    "1203_7000_0001_0001": "AMS4 Filament speed and length error: The slot 1 filament odometry may be faulty.",
    "1200_7100_0001_0001": "AMS1 Filament speed and length error: The slot 2 filament odometry may be faulty.",
    "1201_7100_0001_0001": "AMS2 Filament speed and length error: The slot 2 filament odometry may be faulty.",
    "1202_7100_0001_0001": "AMS3 Filament speed and length error: The slot 2 filament odometry may be faulty.",
    "1203_7100_0001_0001": "AMS4 Filament speed and length error: The slot 2 filament odometry may be faulty.",
    "1200_7200_0001_0001": "AMS1 Filament speed and length error: The slot 3 filament odometry may be faulty.",
    "1201_7200_0001_0001": "AMS2 Filament speed and length error: The slot 3 filament odometry may be faulty.",
    "1202_7200_0001_0001": "AMS3 Filament speed and length error: The slot 3 filament odometry may be faulty.",
    "1203_7200_0001_0001": "AMS4 Filament speed and length error: The slot 3 filament odometry may be faulty.",
    "1200_7300_0001_0001": "AMS1 Filament speed and length error: The slot 4 filament odometry may be faulty.",
    "1201_7300_0001_0001": "AMS2 Filament speed and length error: The slot 4 filament odometry may be faulty.",
    "1202_7300_0001_0001": "AMS3 Filament speed and length error: The slot 4 filament odometry may be faulty.",
    "1203_7300_0001_0001": "AMS4 Filament speed and length error: The slot 4 filament odometry may be faulty.",
    "12FF_2000_0002_0004": "Please pull out the filament on the spool holder from the extruder.",
    "1200_2400_0002_0001": "Filament may be broken in the tool head.",
    "1200_2500_0002_0001": "Failed to extrude the filament and the extruder may be clogged.",
    "0500_0100_0003_0007": "Unable to record time-lapse photography without MicroSD card inserted.",
    "0500_0300_0002_0010": "forward coredump, it is recovering.",
    "0500_0300_0002_0011": "upgrade coredump, it is recovering.",
    "0500_0300_0002_0012": "ipcam coredump, it is recovering.",
    "0500_0300_0002_0016": "device_manager coredump, it is recovering.",
    "0500_0300_0002_0017": "recorder coredump, it is recovering.",
    "0500_0300_0002_0018": "security coredump, it is recovering.",
    "0500_0300_0002_0013": "xcam coredump, it is recovering.",
    "0500_0300_0002_0014": "bbl_screen coredump, it is recovering.",
    "0500_0300_0002_0015": "device_gate coredump, it is recovering.",
    "0500_0400_0003_0008": "The door is detected to be open.",
    "0500_0100_0003_0005": "MicroSD Card error: please reinsert, format or replace it.",
    "0500_0100_0003_0006": "Unformatted MicroSD Card: please format it.",
    "07FF_2000_0002_0001": "External filament has run out; please load a new filament.",
    "07FF_2000_0002_0002": "External filament is missing; please load a new filament.",
    "07FF_2000_0002_0004": "Please pull the external filament from the extruder.",
    "0300_0300_0001_0001": "The speed of the nozzle fan is too slow or stopped. It may be stuck or the connector is not plugged in properly.",
    "0300_0100_0003_0008": "The temperature of the heated bed exceeds the limit and automatically adjusts to the limit temperature.",
    "0300_2000_0001_0003": "X axis homing abnormal: the timing belt may be loose.",
    "0300_2000_0001_0004": "Y axis homing abnormal: the timing belt may be loose.",
    "0C00_0300_0002_0004": "First layer inspection is not supported for the current print job.",
    "0C00_0300_0003_000D": "Some objects may have fallen down, or the extruder is not extruding normally. Please check and decide if the printing should be stopped.",
    "0C00_0300_0003_0008": "Possible spaghetti defects were detected. Please check the print quality and decide if the job should be stopped.",
    "0500_0200_0002_0002": "Device login failed; please check your account information.",
    "0500_0200_0002_0003": "Failed to connect internet; please check the network connection.",
    "0500_0200_0002_0004": "Unauthorized user: please check your account information.",
    "0500_0200_0002_0005": "Failed to connect internet; please check the network connection.",
    "0500_0200_0002_0007": "Liveview service login failed; please check your network connection.",
    "0500_0300_0001_0001": "The MC module is malfunctioning; please restart the device or check device cable connection.",
    "0500_0300_0001_000A": "System state is abnormal; please restore factory settings.",
    "0500_0300_0001_000B": "The screen is malfunctioning; please restart the device.",
    "0300_2000_0001_0001": "X axis homing abnormal: please check if the tool head is stuck or the carbon rod has too much resistance.",
    "0300_2000_0001_0002": "Y axis homing abnormal: please check if the tool head is stuck or the Y carriage has too much resistance.",
    "0300_4000_0002_0001": "Data transmission over the serial port is abnormal; the software system may be faulty.",
    "0300_4100_0001_0001": "The system voltage is unstable; triggering the power failure protection function.",
    "0500_0200_0002_0001": "Failed to connect internet. Please check the network connection.",
    "0300_0400_0002_0001": "The speed of the part cooling fan is too slow or stopped. It may be stuck or the connector is not plugged in properly.",
    "0500_0100_0002_0002": "USB camera is not connected. Please check video camera cable connection.",
    "0500_0300_0002_000C": "Wireless hardware error: please turn off/on WiFi or restart the device.",
    "0C00_0300_0003_000B": "Inspecting the first layer: please wait a moment.",
    "0C00_0200_0002_0004": "Nozzle height seems too low. Please check if the nozzle is worn or tilted. Re-calibrate Lidar if the nozzle has been replaced.",
    "0C00_0300_0003_0007": "Possible first layer defects have been detected. Please check the first layer quality and decide if the job should be stopped.",
    "0C00_0100_0001_000B": "Failed to calibrate Micro Lidar. Please make sure the calibration chart is clean and not occluded, and run machine calibration again.",
    "0C00_0200_0002_0002": "The horizontal laser line is too wide. Please check if the heatbed is dirty.",
    "0C00_0200_0002_0008": "The vertical laser line is too wide. Please check if the heatbed is dirty.",
    "0300_0A00_0001_0001": "Heatbed force sensor 1 is too sensitive. It may be stuck between the strain arm and heatbed support, or the adjusting screw may be too tight.",
    "0300_0A00_0001_0004": "An external disturbance was detected on force sensor 1. The heatbed plate may have touched something outside the heatbed.",
    "0300_0A00_0001_0005": "Force sensor 1 detected unexpected continuous force. The heatbed may be stuck, or the analog front end may be broken.",
    "0300_0B00_0001_0001": "Heatbed force sensor 2 is too sensitive. It may be stuck between the strain arm and heatbed support, or the adjusting screw may be too tight.",
    "0300_0B00_0001_0004": "An external disturbance was detected on force sensor 2. The heatbed plate may have touched something outside the heatbed.",
    "0300_0B00_0001_0005": "Force sensor 2 detected unexpected continuous force. The heatbed may be stuck, or the analog front end may be broken.",
    "0300_0C00_0001_0001": "Heatbed force sensor 3 is too sensitive. It may be stuck between the strain arm and heatbed support, or the adjusting screw may be too tight.",
    "0300_0C00_0001_0004": "An external disturbance was detected on force sensor 3. The heatbed plate may have touched something outside the heatbed.",
    "0300_0C00_0001_0005": "Force sensor 3 detected unexpected continuous force. The heatbed may be stuck, or the analog front end may be broken.",
    "0C00_0100_0001_000A": "The Micro Lidar LED may be broken.",
    "0500_0400_0002_0007": "The bed temperature exceeds the filament's vitrification temperature, which may cause a nozzle clog. Please keep the front door of the printer open or lower the bed temperature.",
    "0300_1000_0002_0001": "The resonance frequency of the X axis is low. The timing belt may be loose.",
    "0300_1100_0002_0001": "The resonance frequency of the Y axis is low. The timing belt may be loose.",
    "0300_0A00_0001_0002": "The signal of heatbed force sensor 1 is weak. The force sensor may be broken or have poor electric connection.",
    "0300_0A00_0001_0003": "The signal of heatbed force sensor 1 is too weak. The electronic connection to the sensor may be broken.",
    "0300_0B00_0001_0003": "The signal of heatbed force sensor 2 is too weak. The electronic connection to the sensor may be broken.",
    "0300_0B00_0001_0002": "The signal of heatbed force sensor 2 is weak. The force sensor may be broken or have poor electric connection.",
    "0300_0C00_0001_0002": "The signal of heatbed force sensor 3 is weak. The force sensor may be broken or have poor electric connection.",
    "0300_0C00_0001_0003": "The signal of heatbed force sensor 3 is too weak. The electronic connection to the sensor may be broken.",
    "0300_0600_0001_0001": "Motor-A has an open-circuit. There may be a loose connection, or the motor may have failed.",
    "0300_0600_0001_0002": "Motor-A has a short-circuit. It may have failed.",
    "0300_0700_0001_0001": "Motor-B has an open-circuit. The connection may be loose, or the motor may have failed.",
    "0300_0700_0001_0002": "Motor-B has a short-circuit. It may have failed.",
    "0300_0800_0001_0001": "Motor-Z has an open-circuit. The connection may be loose, or the motor may have failed.",
    "0300_0800_0001_0002": "Motor-Z has a short-circuit. It may have failed.",
    "0300_0900_0001_0001": "Motor-E has an open-circuit. The connection may be loose, or the motor may have failed.",
    "0300_0900_0001_0002": "Motor-E has a short-circuit. It may have failed.",
    "0C00_0100_0002_0007": "Micro Lidar laser parameters are drifted. Please re-calibrate your printer.",
    "0C00_0300_0002_000C": "The build plate localization marker is not detected. Please check if the build plate is aligned correctly.",
    "0300_0600_0001_0003": "The resistance of Motor-A is abnormal, the motor may have failed.",
    "0300_0700_0001_0003": "The resistance of Motor-B is abnormal, the motor may have failed.",
    "0300_0800_0001_0003": "The resistance of Motor-Z is abnormal, the motor may have failed.",
    "0300_0900_0001_0003": "The resistance of Motor-E is abnormal, the motor may have failed.",
    "0300_1300_0001_0001": "The current sensor of Motor-A is abnormal. This may be caused by a failure of the hardware sampling circuit.",
    "0300_1400_0001_0001": "The current sensor of Motor-B is abnormal. This may be caused by a failure of the hardware sampling circuit.",
    "0300_1500_0001_0001": "The current sensor of Motor-Z is abnormal. This may be caused by a failure of the hardware sampling circuit.",
    "0300_1600_0001_0001": "The current sensor of Motor-E is abnormal. This may be caused by a failure of the hardware sampling circuit.",
    "0C00_0200_0002_0009": "The vertical laser is not bright enough at homing position. Please clean or replace heatbed if this message appears repeatedly.",
    "0C00_0300_0003_000E": "Your printer seems to be printing without extruding.",
    "0C00_0300_0003_000F": "Your nozzle seems to be covered with jammed or clogged material.",
    "0C00_0200_0002_0006": "Nozzle height seems too high. Please check if there is filament residual attached to the nozzle.",
    "0C00_0300_0001_0009": "The first layer inspection module rebooted abnormally. The inspection result may be inaccurate.",
    "0C00_0300_0001_000A": "Your printer is in factory mode. Please contact Technical Support.",
    "0C00_0300_0002_0001": "Filament exposure metering failed because laser reflection is too weak on this material. First layer inspection may be inaccurate.",
    "0C00_0300_0002_0002": "First layer inspection terminated due to abnormal Lidar data.",
    "0C00_0300_0002_0005": "First layer inspection timed out abnormally, and the current results may be inaccurate.",
    "0C00_0200_0002_0007": "The vertical laser is not lit. Please check if it's covered or hardware connection is normal.",
    "0C00_0100_0001_0004": "The Micro Lidar camera lens seems to be dirty. Please clean the lens.",
    "0C00_0100_0001_0005": "Micro Lidar camera OTP parameter is abnormal. Please contact after-sales.",
    "0C00_0100_0001_0009": "The chamber camera lens seems to be dirty. Please clean the lens.",
    "0C00_0100_0002_0002": "The Micro Lidar camera is malfunctioning and related functions may fail. Please contact after-sales if this message keeps appearing in multiple prints.",
    "0C00_0100_0002_0006": "Micro Lidar camera extrinsic parameters are abnormal. Please enable flowrate calibration in your next print.",
    "0C00_0200_0001_0001": "The horizontal laser is not lit. Please check if it's covered or hardware connection is normal.",
    "0C00_0200_0001_0005": "A new Micro Lidar was detected. Please calibrate it on Calibration page before use.",
    "0C00_0200_0002_0003": "The horizontal laser is not bright enough at homing position. Please clean or replace heatbed if this message appears repeatedly.",
    "0C00_0100_0001_0001": "The Micro Lidar camera is offline. Please check the hardware connection.",
    "0C00_0100_0001_0003": "Synchronization between the Micro Lidar camera and MCU is abnormal. Please restart your printer.",
    "0500_0300_0001_0009": "A system hang occurred. It has been recovered by automatic restart.",
    "0500_0300_0002_000D": "The SD Card controller is malfunctioning.",
    "0500_0300_0003_0007": "A system panic occurred. It has been recovered by automatic restart.",
    "0500_0400_0001_0001": "Failed to download print job. Please check your network connection.",
    "0500_0400_0001_0002": "Failed to report print state. Please check your network connection.",
    "0500_0400_0001_0003": "The content of print file is unreadable. Please resend the print job.",
    "0500_0400_0001_0004": "The print file is unauthorized.",
    "0500_0400_0001_0006": "Failed to resume previous print.",
    "0500_0300_0001_0002": "The toolhead is malfunctioning. Please restart the device.",
    "0500_0300_0001_0003": "The AMS module is malfunctioning. Please restart the device.",
    "0500_0300_0001_0005": "Internal service is malfunctioning. Please restart the device.",
    "0500_0300_0001_0006": "A system panic occurred. Please restart the device.",
    "0500_0300_0001_0008": "A system hang occurred. Please restart the device.",
    "0300_1200_0002_0001": "The front cover of the toolhead fell off.",
    "0500_0100_0002_0001": "The media pipeline is malfunctioning.",
    "0500_0100_0002_0003": "USB camera is malfunctioning.",
    "0300_0D00_0001_0002": "Heatbed homing failed. The environmental vibration is too great.",
    "0300_0D00_0001_0003": "The build plate is not placed properly. Please adjust it.",
    "0300_0D00_0001_0004": "The build plate is not placed properly. Please adjust it.",
    "0300_0D00_0001_0005": "The build plate is not placed properly. Please adjust it.",
    "0300_0D00_0001_0006": "The build plate is not placed properly. Please adjust it.",
    "0300_0D00_0001_0007": "The build plate is not placed properly. Please adjust it.",
    "0300_0D00_0001_0008": "The build plate is not placed properly. Please adjust it.",
    "0300_0D00_0001_0009": "The build plate is not placed properly. Please adjust it.",
    "0300_0D00_0001_000A": "The build plate is not placed properly. Please adjust it.",
    "0300_0100_0001_0001": "The heatbed temperature is abnormal; the heater may have a short circuit.",
    "0300_0100_0001_0002": "The heatbed temperature is abnormal; the heater may have an open circuit, or the thermal switch may be open.",
    "0300_0100_0001_0003": "The heatbed temperature is abnormal; the heater is over temperature.",
    "0300_0200_0001_0001": "The nozzle temperature is abnormal; the heater may have a short circuit.",
    "0300_0200_0001_0002": "The nozzle temperature is abnormal; the heater may have an open circuit.",
    "0300_0200_0001_0003": "The nozzle temperature is abnormal; the heater is over temperature.",
    "0300_0500_0001_0001": "The motor driver is overheating. Its radiator may be loose, or its cooling fan may be damaged.",
    "0300_0100_0001_0006": "The heatbed temperature is abnormal; the sensor may have a short circuit.",
    "0300_0100_0001_0007": "The heatbed temperature is abnormal; the sensor may have an open circuit.",
    "0300_0200_0001_0007": "The nozzle temperature is abnormal; the sensor may have an open circuit.",
}
# UNIQUE_ID=wy2WtJ2q

# These errors cover those that are AMS and/or slot specific.
# 070X_xYxx_xxxx_xxxx = AMS X (0 based index) Slot Y (0 based index) has the error.
# UNIQUE_ID=dxeWW5n6
HMS_AMS_ERRORS = {
    "0700_3000_0002_0002": "The RFID-tag on AMS1 Slot1 is damaged or the it's content cannot be identified.",
    "0700_3100_0002_0002": "The RFID-tag on AMS1 Slot2 is damaged or the it's content cannot be identified.",
    "0700_3200_0002_0002": "The RFID-tag on AMS1 Slot3 is damaged or the it's content cannot be identified.",
    "0700_3300_0002_0002": "The RFID-tag on AMS1 Slot4 is damaged or the it's content cannot be identified.",
    "0701_3000_0002_0002": "The RFID-tag on AMS2 Slot1 is damaged or the it's content cannot be identified.",
    "0701_3100_0002_0002": "The RFID-tag on AMS2 Slot2 is damaged or the it's content cannot be identified.",
    "0701_3200_0002_0002": "The RFID-tag on AMS2 Slot3 is damaged or the it's content cannot be identified.",
    "0701_3300_0002_0002": "The RFID-tag on AMS2 Slot4 is damaged or the it's content cannot be identified.",
    "0702_3000_0002_0002": "The RFID-tag on AMS3 Slot1 is damaged or the it's content cannot be identified.",
    "0702_3100_0002_0002": "The RFID-tag on AMS3 Slot2 is damaged or the it's content cannot be identified.",
    "0702_3200_0002_0002": "The RFID-tag on AMS3 Slot3 is damaged or the it's content cannot be identified.",
    "0702_3300_0002_0002": "The RFID-tag on AMS3 Slot4 is damaged or the it's content cannot be identified.",
    "0703_3000_0002_0002": "The RFID-tag on AMS4 Slot1 is damaged or the it's content cannot be identified.",
    "0703_3100_0002_0002": "The RFID-tag on AMS4 Slot2 is damaged or the it's content cannot be identified.",
    "0703_3200_0002_0002": "The RFID-tag on AMS4 Slot3 is damaged or the it's content cannot be identified.",
    "0703_3300_0002_0002": "The RFID-tag on AMS4 Slot4 is damaged or the it's content cannot be identified.",
    "0700_4000_0002_0004": "The filament buffer signal is abnormal; the spring may be stuck or the filament may be tangle.",
    "0700_2000_0002_0002": "AMS1 Slot1 is empty; please load a new filament.",
    "0700_2100_0002_0002": "AMS1 Slot2 is empty; please load a new filament.",
    "0700_2200_0002_0002": "AMS1 Slot3 is empty; please load a new filament.",
    "0700_2300_0002_0002": "AMS1 Slot4 is empty; please load a new filament.",
    "0701_2000_0002_0002": "AMS2 Slot1 is empty; please load a new filament.",
    "0701_2100_0002_0002": "AMS2 Slot2 is empty; please load a new filament.",
    "0701_2200_0002_0002": "AMS2 Slot3 is empty; please load a new filament.",
    "0701_2300_0002_0002": "AMS2 Slot4 is empty; please load a new filament.",
    "0702_2000_0002_0002": "AMS3 Slot1 is empty; please load a new filament.",
    "0702_2100_0002_0002": "AMS3 Slot2 is empty; please load a new filament.",
    "0702_2200_0002_0002": "AMS3 Slot3 is empty; please load a new filament.",
    "0702_2300_0002_0002": "AMS3 Slot4 is empty; please load a new filament.",
    "0703_2000_0002_0002": "AMS4 Slot1 is empty; please load a new filament.",
    "0703_2100_0002_0002": "AMS4 Slot2 is empty; please load a new filament.",
    "0703_2200_0002_0002": "AMS4 Slot3 is empty; please load a new filament.",
    "0703_2300_0002_0002": "AMS4 Slot4 is empty; please load a new filament.",
    "0700_2000_0003_0001": "AMS1 Slot1 filament has run out. Please wait while old filament is purged.",
    "0700_2100_0003_0001": "AMS1 Slot2 filament has run out. Please wait while old filament is purged.",
    "0700_2200_0003_0001": "AMS1 Slot3 filament has run out. Please wait while old filament is purged.",
    "0700_2000_0003_0002": "AMS1 Slot1 filament has run out and automatically switched to the slot with the same filament.",
    "0700_2100_0003_0002": "AMS1 Slot2 filament has run out and automatically switched to the slot with the same filament.",
    "0700_2200_0003_0002": "AMS1 Slot3 filament has run out and automatically switched to the slot with the same filament.",
    "0702_5000_0002_0001": "AMS3 communication is abnormal; please check the connection cable.",
    "0703_5000_0002_0001": "AMS4 communication is abnormal; please check the connection cable.",
    "0700_6000_0002_0001": "The AMS1 slot1 is overloaded. The filament may be tangled or the spool may be stuck.",
    "0701_6000_0002_0001": "The AMS2 slot1 is overloaded. The filament may be tangled or the spool may be stuck.",
    "0702_6000_0002_0001": "The AMS3 slot1 is overloaded. The filament may be tangled or the spool may be stuck.",
    "0703_6000_0002_0001": "The AMS4 slot1 is overloaded. The filament may be tangled or the spool may be stuck.",
    "0700_6100_0002_0001": "The AMS1 slot2 is overloaded. The filament may be tangled or the spool may be stuck.",
    "0701_6100_0002_0001": "The AMS2 slot2 is overloaded. The filament may be tangled or the spool may be stuck.",
    "0702_6100_0002_0001": "The AMS3 slot2 is overloaded. The filament may be tangled or the spool may be stuck.",
    "0703_6100_0002_0001": "The AMS4 slot2 is overloaded. The filament may be tangled or the spool may be stuck.",
    "0700_6200_0002_0001": "The AMS1 slot3 is overloaded. The filament may be tangled or the spool may be stuck.",
    "0701_6200_0002_0001": "The AMS2 slot3 is overloaded. The filament may be tangled or the spool may be stuck.",
    "0702_6200_0002_0001": "The AMS3 slot3 is overloaded. The filament may be tangled or the spool may be stuck.",
    "0703_6200_0002_0001": "The AMS4 slot3 is overloaded. The filament may be tangled or the spool may be stuck.",
    "0700_6300_0002_0001": "The AMS1 slot4 is overloaded. The filament may be tangled or the spool may be stuck.",
    "0701_6300_0002_0001": "The AMS2 slot4 is overloaded. The filament may be tangled or the spool may be stuck.",
    "0702_6300_0002_0001": "The AMS3 slot4 is overloaded. The filament may be tangled or the spool may be stuck.",
    "0703_6300_0002_0001": "The AMS4 slot4 is overloaded. The filament may be tangled or the spool may be stuck.",
    "0700_2000_0002_0004": "AMS1 Slot1 filament may be broken in the tool head.",
    "0702_2000_0002_0004": "AMS3 Slot1 filament may be broken in the tool head.",
    "0703_2000_0002_0004": "AMS4 Slot1 filament may be broken in the tool head.",
    "0700_2100_0002_0004": "AMS1 Slot2 filament may be broken in the tool head.",
    "0701_2100_0002_0004": "AMS2 Slot2 filament may be broken in the tool head.",
    "0702_2100_0002_0004": "AMS3 Slot2 filament may be broken in the tool head.",
    "0703_2100_0002_0004": "AMS4 Slot2 filament may be broken in the tool head.",
    "0700_2200_0002_0004": "AMS1 Slot3 filament may be broken in the tool head.",
    "0701_2200_0002_0004": "AMS2 Slot3 filament may be broken in the tool head.",
    "0702_2200_0002_0004": "AMS3 Slot3 filament may be broken in the tool head.",
    "0703_2200_0002_0004": "AMS4 Slot3 filament may be broken in the tool head.",
    "0700_2300_0002_0004": "AMS1 Slot4 filament may be broken in the tool head.",
    "0701_2300_0002_0004": "AMS2 Slot4 filament may be broken in the tool head.",
    "0702_2300_0002_0004": "AMS3 Slot4 filament may be broken in the tool head.",
    "0703_2300_0002_0004": "AMS4 Slot4 filament may be broken in the tool head.",
    "0702_2000_0002_0005": "AMS3 Slot1 filament has run out, and purging the old filament went abnormally; please check whether the filament is stuck in the tool head.",
    "0703_2000_0002_0005": "AMS4 Slot1 filament has run out, and purging the old filament went abnormally; please check whether the filament is stuck in the tool head.",
    "0702_2100_0002_0005": "AMS3 Slot2 filament has run out, and purging the old filament went abnormally; please check whether the filament is stuck in the tool head.",
    "0703_2100_0002_0005": "AMS4 Slot2 filament has run out, and purging the old filament went abnormally; please check whether the filament is stuck in the tool head.",
    "0702_2200_0002_0005": "AMS3 Slot3 filament has run out, and purging the old filament went abnormally; please check whether the filament is stuck in the tool head.",
    "0703_2200_0002_0005": "AMS4 Slot3 filament has run out, and purging the old filament went abnormally; please check whether the filament is stuck in the tool head.",
    "0702_2300_0002_0005": "AMS3 Slot4 filament has run out, and purging the old filament went abnormally; please check whether the filament is stuck in the tool head.",
    "0703_2300_0002_0005": "AMS4 Slot4 filament has run out, and purging the old filament went abnormally; please check whether the filament is stuck in the tool head.",
    "0701_2000_0003_0002": "AMS2 Slot1 filament has run out and automatically switched to the slot with the same filament.",
    "0702_2000_0003_0002": "AMS3 Slot1 filament has run out and automatically switched to the slot with the same filament.",
    "0703_2000_0003_0002": "AMS4 Slot1 filament has run out and automatically switched to the slot with the same filament.",
    "0701_2100_0003_0002": "AMS2 Slot2 filament has run out and automatically switched to the slot with the same filament.",
    "0702_2100_0003_0002": "AMS3 Slot2 filament has run out and automatically switched to the slot with the same filament.",
    "0703_2100_0003_0002": "AMS4 Slot2 filament has run out and automatically switched to the slot with the same filament.",
    "0701_2200_0003_0002": "AMS2 Slot3 filament has run out and automatically switched to the slot with the same filament.",
    "0702_2200_0003_0002": "AMS3 Slot3 filament has run out and automatically switched to the slot with the same filament.",
    "0703_2200_0003_0002": "AMS4 Slot3 filament has run out and automatically switched to the slot with the same filament.",
    "0700_2300_0003_0002": "AMS1 Slot4 filament has run out and automatically switched to the slot with the same filament.",
    "0701_2300_0003_0002": "AMS2 Slot4 filament has run out and automatically switched to the slot with the same filament.",
    "0702_2300_0003_0002": "AMS3 Slot4 filament has run out and automatically switched to the slot with the same filament.",
    "0703_2300_0003_0002": "AMS4 Slot4 filament has run out and automatically switched to the slot with the same filament.",
    "0700_4500_0002_0001": "The filament cutter sensor is malfunctioning; please check whether the connector is properly plugged in.",
    "0700_5000_0002_0001": "AMS1 communication is abnormal; please check the connection cable.",
    "0701_5000_0002_0001": "AMS2 communication is abnormal; please check the connection cable.",
    "0701_2000_0002_0004": "AMS2 Slot1 filament may be broken in the tool head.",
    "0700_2000_0002_0005": "AMS1 Slot1 filament has run out, and purging the old filament went abnormally; please check whether the filament is stuck in the tool head.",
    "0701_2000_0002_0005": "AMS2 Slot1 filament has run out, and purging the old filament went abnormally; please check whether the filament is stuck in the tool head.",
    "0700_2100_0002_0005": "AMS1 Slot2 filament has run out, and purging the old filament went abnormally; please check whether the filament is stuck in the tool head.",
    "0701_2100_0002_0005": "AMS2 Slot2 filament has run out, and purging the old filament went abnormally; please check whether the filament is stuck in the tool head.",
    "0700_2200_0002_0005": "AMS1 Slot3 filament has run out, and purging the old filament went abnormally; please check whether the filament is stuck in the tool head.",
    "0701_2200_0002_0005": "AMS2 Slot3 filament has run out, and purging the old filament went abnormally; please check whether the filament is stuck in the tool head.",
    "0700_2300_0002_0005": "AMS1 Slot4 filament has run out, and purging the old filament went abnormally; please check whether the filament is stuck in the tool head.",
    "0701_2300_0002_0005": "AMS2 Slot4 filament has run out, and purging the old filament went abnormally; please check whether the filament is stuck in the tool head.",
    "0700_5100_0003_0001": "The AMS is disabled; please load filament from spool holder.",
    "0700_2000_0002_0001": "AMS1 Slot1 filament has run out. Please insert a new filament.",
    "0700_2000_0002_0003": "AMS1 Slot1's filament may be broken in AMS.",
    "0700_2100_0002_0001": "AMS1 Slot2 filament has run out. Please insert a new filament.",
    "0700_2100_0002_0003": "AMS1 Slot2's filament may be broken in AMS.",
    "0700_2200_0002_0001": "AMS1 Slot3 filament has run out. Please insert a new filament.",
    "0700_2200_0002_0003": "AMS1 Slot3's filament may be broken in AMS.",
    "0700_2300_0002_0001": "AMS1 Slot4 filament has run out. Please insert a new filament.",
    "0700_2300_0002_0003": "AMS1 Slot4's filament may be broken in AMS.",
    "0701_2000_0002_0001": "AMS2 Slot1 filament has run out. Please insert a new filament.",
    "0701_2000_0002_0003": "AMS2 Slot1's filament may be broken in AMS.",
    "0701_2100_0002_0001": "AMS2 Slot2 filament has run out. Please insert a new filament.",
    "0701_2100_0002_0003": "AMS2 Slot2's filament may be broken in AMS.",
    "0701_2200_0002_0001": "AMS2 Slot3 filament has run out. Please insert a new filament.",
    "0701_2200_0002_0003": "AMS2 Slot3's filament may be broken in AMS.",
    "0701_2300_0002_0001": "AMS2 Slot4 filament has run out. Please insert a new filament.",
    "0701_2300_0002_0003": "AMS2 Slot4's filament may be broken in AMS.",
    "0702_2000_0002_0001": "AMS3 Slot1 filament has run out. Please insert a new filament.",
    "0702_2000_0002_0003": "AMS3 Slot1's filament may be broken in AMS.",
    "0702_2100_0002_0001": "AMS3 Slot2 filament has run out. Please insert a new filament.",
    "0702_2100_0002_0003": "AMS3 Slot2's filament may be broken in AMS.",
    "0702_2200_0002_0001": "AMS3 Slot3 filament has run out. Please insert a new filament.",
    "0702_2200_0002_0003": "AMS3 Slot3's filament may be broken in AMS.",
    "0702_2300_0002_0001": "AMS3 Slot4 filament has run out. Please insert a new filament.",
    "0702_2300_0002_0003": "AMS3 Slot4's filament may be broken in AMS.",
    "0703_2000_0002_0001": "AMS4 Slot1 filament has run out. Please insert a new filament.",
    "0703_2000_0002_0003": "AMS4 Slot1's filament may be broken in AMS.",
    "0703_2100_0002_0001": "AMS4 Slot2 filament has run out. Please insert a new filament.",
    "0703_2100_0002_0003": "AMS4 Slot2's filament may be broken in AMS.",
    "0703_2200_0002_0001": "AMS4 Slot3 filament has run out. Please insert a new filament.",
    "0703_2200_0002_0003": "AMS4 Slot3's filament may be broken in AMS.",
    "0703_2300_0002_0001": "AMS4 Slot4 filament has run out. Please insert a new filament.",
    "0703_2300_0002_0003": "AMS4 Slot4's filament may be broken in AMS.",
    "0701_2000_0003_0001": "AMS2 Slot1 filament has run out. Please wait while old filament is purged.",
    "0702_2000_0003_0001": "AMS3 Slot1 filament has run out. Please wait while old filament is purged.",
    "0703_2000_0003_0001": "AMS4 Slot1 filament has run out. Please wait while old filament is purged.",
    "0701_2100_0003_0001": "AMS2 Slot2 filament has run out. Please wait while old filament is purged.",
    "0702_2100_0003_0001": "AMS3 Slot2 filament has run out. Please wait while old filament is purged.",
    "0703_2100_0003_0001": "AMS4 Slot2 filament has run out. Please wait while old filament is purged.",
    "0701_2200_0003_0001": "AMS2 Slot3 filament has run out. Please wait while old filament is purged.",
    "0702_2200_0003_0001": "AMS3 Slot3 filament has run out. Please wait while old filament is purged.",
    "0703_2200_0003_0001": "AMS4 Slot3 filament has run out. Please wait while old filament is purged.",
    "0700_2300_0003_0001": "AMS1 Slot4 filament has run out. Please wait while old filament is purged.",
    "0701_2300_0003_0001": "AMS2 Slot4 filament has run out. Please wait while old filament is purged.",
    "0702_2300_0003_0001": "AMS3 Slot4 filament has run out. Please wait while old filament is purged.",
    "0703_2300_0003_0001": "AMS4 Slot4 filament has run out. Please wait while old filament is purged.",
    "0703_3000_0003_0003": "RFID cannot be read because of a hardware or structural error.",
    "0703_3100_0001_0001": "The RFID board between AMS4 Slot3 & Slot4 has an error.",
    "0703_3100_0001_0004": "Encryption chip failure.",
    "0703_3100_0003_0003": "RFID cannot be read because of a hardware or structural error.",
    "0703_3500_0001_0001": "The temperature and humidity sensor has an error. The chip may be faulty.",
    "0703_1300_0002_0002": "The AMS4 slot4 motor is overloaded. The filament may be tangled or stuck.",
    "0703_3000_0001_0001": "The RFID board between AMS4 Slot1 & Slot2 has an error.",
    "0703_3000_0001_0004": "Encryption chip failure.",
    "0703_0100_0001_0003": "The AMS4 assist motor torque control is malfunctioning. The current sensor may be faulty.",
    "0703_0100_0001_0004": "The AMS4 assist motor speed control is malfunctioning. The speed sensor may be faulty.",
    "0703_0100_0002_0002": "The AMS4 assist motor is overloaded. The filament may be tangled or stuck.",
    "0703_0200_0001_0001": "AMS4 Filament speed and length error: The filament odometry may be faulty.",
    "0703_1000_0001_0001": "The AMS4 slot1 motor has slipped. The extrusion wheel may be malfunctioning, or the filament may be too thin.",
    "0703_1000_0001_0003": "The AMS4 slot1 motor torque control is malfunctioning. The current sensor may be faulty.",
    "0703_1000_0002_0002": "The AMS4 slot1 motor is overloaded. The filament may be tangled or stuck.",
    "0703_1100_0001_0001": "The AMS4 slot2 motor has slipped. The extrusion wheel may be malfunctioning, or the filament may be too thin.",
    "0703_1100_0001_0003": "The AMS4 slot2 motor torque control is malfunctioning. The current sensor may be faulty.",
    "0703_1100_0002_0002": "The AMS4 slot2 motor is overloaded. The filament may be tangled or stuck.",
    "0703_1200_0001_0001": "The AMS4 slot3 motor has slipped. The extrusion wheel may be malfunctioning, or the filament may be too thin.",
    "0703_1200_0001_0003": "The AMS4 slot3 motor torque control is malfunctioning. The current sensor may be faulty.",
    "0703_1200_0002_0002": "The AMS4 slot3 motor is overloaded. The filament may be tangled or stuck.",
    "0703_1300_0001_0001": "The AMS4 slot4 motor has slipped. The extrusion wheel may be malfunctioning, or the filament may be too thin.",
    "0703_1300_0001_0003": "The AMS4 slot4 motor torque control is malfunctioning. The current sensor may be faulty.",
    "0702_3000_0003_0003": "RFID cannot be read because of a hardware or structural error.",
    "0702_3100_0001_0001": "The RFID board between AMS3 Slot3 & Slot4 has an error.",
    "0702_3100_0001_0004": "Encryption chip failure.",
    "0702_3100_0003_0003": "RFID cannot be read because of a hardware or structural error.",
    "0702_3500_0001_0001": "The temperature and humidity sensor has an error. The chip may be faulty.",
    "0703_0100_0001_0001": "The AMS4 assist motor has slipped. The extrusion wheel may be worn down, or the filament may be too thin.",
    "0702_3000_0001_0001": "The RFID board between AMS3 Slot1 & Slot2 has an error.",
    "0702_3000_0001_0004": "Encryption chip failure.",
    "0702_0100_0002_0002": "The AMS3 assist motor is overloaded. The filament may be tangled or stuck.",
    "0702_0200_0001_0001": "AMS3 Filament speed and length error: The filament odometry may be faulty.",
    "0702_1000_0001_0001": "The AMS3 slot1 motor has slipped. The extrusion wheel may be malfunctioning, or the filament may be too thin.",
    "0702_1000_0001_0003": "The AMS3 slot1 motor torque control is malfunctioning. The current sensor may be faulty.",
    "0702_1000_0002_0002": "The AMS3 slot1 motor is overloaded. The filament may be tangled or stuck.",
    "0702_1100_0001_0001": "The AMS3 slot2 motor has slipped. The extrusion wheel may be malfunctioning, or the filament may be too thin.",
    "0702_1100_0001_0003": "The AMS3 slot2 motor torque control is malfunctioning. The current sensor may be faulty.",
    "0702_1100_0002_0002": "The AMS3 slot2 motor is overloaded. The filament may be tangled or stuck.",
    "0702_1200_0001_0001": "The AMS3 slot3 motor has slipped. The extrusion wheel may be malfunctioning, or the filament may be too thin.",
    "0702_1200_0001_0003": "The AMS3 slot3 motor torque control is malfunctioning. The current sensor may be faulty.",
    "0702_1200_0002_0002": "The AMS3 slot3 motor is overloaded. The filament may be tangled or stuck.",
    "0702_1300_0001_0001": "The AMS3 slot4 motor has slipped. The extrusion wheel may be malfunctioning, or the filament may be too thin.",
    "0702_1300_0001_0003": "The AMS3 slot4 motor torque control is malfunctioning. The current sensor may be faulty.",
    "0702_1300_0002_0002": "The AMS3 slot4 motor is overloaded. The filament may be tangled or stuck.",
    "0701_3100_0001_0001": "The RFID board between AMS2 Slot3 & Slot4 has an error.",
    "0701_3100_0001_0004": "Encryption chip failure.",
    "0701_3100_0003_0003": "RFID cannot be read because of a hardware or structural error.",
    "0701_3500_0001_0001": "The temperature and humidity sensor has an error. The chip may be faulty.",
    "0702_0100_0001_0001": "The AMS3 assist motor has slipped. The extrusion wheel may be worn down, or the filament may be too thin.",
    "0702_0100_0001_0003": "The AMS3 assist motor torque control is malfunctioning. The current sensor may be faulty.",
    "0702_0100_0001_0004": "The AMS3 assist motor speed control is malfunctioning. The speed sensor may be faulty.",
    "0701_3000_0001_0001": "The RFID board between AMS2 Slot1 & Slot2 has an error.",
    "0701_3000_0001_0004": "Encryption chip failure.",
    "0701_3000_0003_0003": "RFID cannot be read because of a hardware or structural error.",
    "0701_0100_0002_0002": "The AMS2 assist motor is overloaded. The filament may be tangled or stuck.",
    "0701_0200_0001_0001": "AMS2 Filament speed and length error: The filament odometry may be faulty.",
    "0701_1000_0001_0001": "The AMS2 slot1 motor has slipped. The extrusion wheel may be malfunctioning, or the filament may be too thin.",
    "0701_1000_0001_0003": "The AMS2 slot1 motor torque control is malfunctioning. The current sensor may be faulty.",
    "0701_1000_0002_0002": "The AMS2 slot1 motor is overloaded. The filament may be tangled or stuck.",
    "0701_1100_0001_0001": "The AMS2 slot2 motor has slipped. The extrusion wheel may be malfunctioning, or the filament may be too thin.",
    "0701_1100_0001_0003": "The AMS2 slot2 motor torque control is malfunctioning. The current sensor may be faulty.",
    "0701_1100_0002_0002": "The AMS2 slot2 motor is overloaded. The filament may be tangled or stuck.",
    "0701_1200_0001_0001": "The AMS2 slot3 motor has slipped. The extrusion wheel may be malfunctioning, or the filament may be too thin.",
    "0701_1200_0001_0003": "The AMS2 slot3 motor torque control is malfunctioning. The current sensor may be faulty.",
    "0701_1200_0002_0002": "The AMS2 slot3 motor is overloaded. The filament may be tangled or stuck.",
    "0701_1300_0001_0001": "The AMS2 slot4 motor has slipped. The extrusion wheel may be malfunctioning, or the filament may be too thin.",
    "0701_1300_0001_0003": "The AMS2 slot4 motor torque control is malfunctioning. The current sensor may be faulty.",
    "0701_1300_0002_0002": "The AMS2 slot4 motor is overloaded. The filament may be tangled or stuck.",
    "0700_3100_0001_0004": "Encryption chip failure.",
    "0700_3100_0003_0003": "RFID cannot be read because of a hardware or structural error.",
    "0700_3500_0001_0001": "The temperature and humidity sensor has an error. The chip may be faulty.",
    "0700_4000_0002_0001": "The filament buffer position signal lost: the cable or position sensor may be malfunctioning.",
    "0700_4000_0002_0002": "The filament buffer position signal error: the position sensor may be malfunctioning.",
    "0700_4000_0002_0003": "The AMS Hub communication is abnormal, the cable may be not well connected.",
    "0700_4500_0002_0002": "The filament cutter's cutting distance is too large. The XY motor may lose steps.",
    "0700_4500_0002_0003": "The filament cutter handle has not released. The handle or blade may be stuck.",
    "0701_0100_0001_0001": "The AMS2 assist motor has slipped. The extrusion wheel may be worn down, or the filament may be too thin.",
    "0701_0100_0001_0003": "The AMS2 assist motor torque control is malfunctioning. The current sensor may be faulty.",
    "0701_0100_0001_0004": "The AMS2 assist motor speed control is malfunctioning. The speed sensor may be faulty.",
    "0700_3000_0001_0001": "The RFID board between AMS1 Slot1 & Slot2 has an error.",
    "0700_3000_0001_0004": "Encryption chip failure.",
    "0700_3000_0003_0003": "RFID cannot be read because of a hardware or structural error.",
    "0700_3100_0001_0001": "The RFID board between AMS1 Slot3 & Slot4 has an error.",
    "0700_0200_0001_0001": "AMS1 Filament speed and length error: The filament odometry may be faulty.",
    "0700_1000_0001_0001": "The AMS1 slot1 motor has slipped. The extrusion wheel may be malfunctioning, or the filament may be too thin.",
    "0700_1000_0001_0003": "The AMS1 slot1 motor torque control is malfunctioning. The current sensor may be faulty.",
    "0700_1000_0002_0002": "The AMS1 slot1 motor is overloaded. The filament may be tangled or stuck.",
    "0700_1100_0001_0001": "The AMS1 slot2 motor has slipped. The extrusion wheel may be malfunctioning, or the filament may be too thin.",
    "0700_1100_0001_0003": "The AMS1 slot2 motor torque control is malfunctioning. The current sensor may be faulty.",
    "0700_1100_0002_0002": "The AMS1 slot2 motor is overloaded. The filament may be tangled or stuck.",
    "0700_1200_0001_0001": "The AMS1 slot3 motor has slipped. The extrusion wheel may be malfunctioning, or the filament may be too thin.",
    "0700_1200_0001_0003": "The AMS1 slot3 motor torque control is malfunctioning. The current sensor may be faulty.",
    "0700_1200_0002_0002": "The AMS1 slot3 motor is overloaded. The filament may be tangled or stuck.",
    "0700_1300_0001_0001": "The AMS1 slot4 motor has slipped. The extrusion wheel may be malfunctioning, or the filament may be too thin.",
    "0700_1300_0001_0003": "The AMS1 slot4 motor torque control is malfunctioning. The current sensor may be faulty.",
    "0700_1300_0002_0002": "The AMS1 slot4 motor is overloaded. The filament may be tangled or stuck.",
    "0700_0100_0001_0001": "The AMS1 assist motor has slipped. The extrusion wheel may be worn down, or the filament may be too thin.",
    "0700_0100_0001_0003": "The AMS1 assist motor torque control is malfunctioning. The current sensor may be faulty.",
    "0700_0100_0001_0004": "The AMS1 assist motor speed control is malfunctioning. The speed sensor may be faulty.",
    "0700_0100_0002_0002": "The AMS1 assist motor is overloaded. The filament may be tangled or stuck.",
}
# UNIQUE_ID=ARxX6kr9

# UNIQUE_ID=ZEJTS2b8
PRINT_ERROR_ERRORS = {
    "1201_8007": "Failed to extrude the filament. The extruder may be clogged or the filament may be stuck; please refer to HMS. After troubleshooting, click 'Retry' button.",
    "1202_8007": "Failed to extrude the filament. The extruder may be clogged or the filament may be stuck; please refer to HMS. After troubleshooting, click 'Retry' button.",
    "1203_8007": "Failed to extrude the filament. The extruder may be clogged or the filament may be stuck; please refer to HMS. After troubleshooting, click 'Retry' button.",
    "1201_8014": "Failed to check the filament location in the tool head; please refer to the HMS. After troubleshooting, click the 'Retry' button.",
    "1202_8014": "Failed to check the filament location in the tool head; please refer to the HMS. After troubleshooting, click the 'Retry' button.",
    "0501_4038": "The region settings do not match the printer; please check the printer's region settings.",
    "1201_8016": "The extruder is not extruding normally; please refer to the HMS. After trouble shooting. If the defects are acceptable, please click 'Retry' button.",
    "1202_8016": "The extruder is not extruding normally; please refer to the HMS. After trouble shooting. If the defects are acceptable, please click 'Retry' button.",
    "1203_8016": "The extruder is not extruding normally; please refer to the HMS. After trouble shooting. If the defects are acceptable, please click 'Retry' button.",
    "0300_8015": "The filament has run out; please load new filament. If the filament is loaded, please select 'Resume'.",
    "0501_4033": "Your APP region is not matched with your printer; please download the APP in the corresponding region and register your account again.",
    "0300_8017": "Foreign objects detected on  hotbed. Please check and clean the hotbed, then tap 'Resume' button to resume the print job.",
    "0514_039": "Device login has expired; please try to bind again.",
    "1000_C001": "High bed temperature may lead to filament clogging in the nozzle. You may open the chamber door.",
    "0300_8010": "The hotend fan speed is abnormal.",
    "1200_8006": "Unable to feed filament into the extruder. This could be due to entangled filament or a stuck spool. If not, please check if the AMS PTFE tube is connected.",
    "1200_8016": "The extruder is not extruding normally. Refer to the Assistant for troubleshooting. There may be defects in this layer, you may resume if the defects are acceptable.",
    "0500_403A": "The current temperature is too low. In order to protect you and your printer. Printing task, moving axis and other operations are disabled. Please move the printer to an environment above 10 celsius degree.",
    "0700_8006": "Unable to feed filament into the extruder. This could be due to entangled filament or a stuck spool. If not, please check if the AMS PTFE tube is connected.",
    "0300_8013": "Printing was paused by the user. You can select 'Resume' to continue printing.",
    "0300_4000": "Printing stopped because homing Z axis failed.",
    "0500_4003": "Printing stopped because the printer was unable to parse the file. Please resend your print job.",
    "0500_4004": "The printer can't receive new print jobs while printing. Resend after the current print finishes.",
    "0700_8002": "The cutter is stuck. Please make sure the cutter handle is out.",
    "0700_8003": "Failed to pull out the filament from the extruder. This might be caused by clogged extruder or filament broken inside the extruder.",
    "0700_8004": "AMS failed to pull back filament. This could be due to a stuck spool or the end of the filament being stuck in the path.",
    "0700_8005": "The AMS failed to send out filament. You can clip the end of your filament flat, and reinsert. If this message persists, please check the PTFE tubes in AMS for any signs of wear and tear.",
    "0700_8007": "Extruding filament failed. The extruder might be clogged.",
    "0700_8010": "The AMS assist motor is overloaded. This could be due to entangled filament or a stuck spool.",
    "0701_8003": "Failed to pull out the filament from the extruder. There may be an extruder clog or broken filament inside the extruder.",
    "0701_8004": "AMS failed to pull back filament. This could be due to a stuck spool or the end of the filament being stuck in the path.",
    "0701_8005": "The AMS failed to send filament. You canclip the end of your filament flat, and reinsert. If this message persists, please check the PTFE tubes in AMS for any signs of wear and tear.",
    "0701_8010": "The AMS assist motor is overloaded. This could be due to entangled filament or a stuck spool.",
    "0700_4001": "The AMS has been disabled for a print, but it still has filament loaded. Please unload the AMS filament , and switch to the spool holder filament for printing.",
    "0500_4014": "Slicing for the print job failed. Please check your settings and restart the print job.",
    "1200_8001": "Cutting the filament failed. Please check to see if the cutter is stuck. Refer to the Assistant for solutions.",
    "1200_8007": "Failed to extrude the filament. This might be caused by clogged extruder or stuck filament. Refer to the Assistant for solutions.",
    "1200_8010": "Filament or spool may be stuck.",
    "1200_8014": "The filament location in the toolhead was not found. Refer to the Assistant for solutions.",
    "1200_8015": "Failed to pull out the filament from the toolhead. Please check if the filament is stuck, or the filament is broken inside the extruder or PTFE tube.",
    "0300_4002": "Printing Stopped because Auto Bed Leveling failed.",
    "0300_8002": "First layer defects were detected by the Micro Lidar. Please check the quality of the printed model before continuing your print.",
    "0300_8003": "Spaghetti defects were detected by the AI Print Monitoring. Please check the quality of the printed model before continuing your print.",
    "0300_8004": "Filament ran out. Please load new filament.",
    "0300_8005": "Toolhead front cover fell off. Please remount the front cover and check to make sure your print is going okay.",
    "0300_8006": "The build plate marker was not detected. Please confirm the build plate is correctly positioned on the heatbed with all four corners aligned, and the maker is clear.",
    "0700_8011": "AMS filament ran out. Please insert a new filament into the same AMS slot.",
    "0701_8011": "AMS filament ran out. Please insert a new filament into the same AMS slot.",
    "0702_8011": "AMS filament ran out. Please insert a new filament into the same AMS slot.",
    "0703_8011": "AMS filament ran out. Please insert a new filament into the same AMS slot.",
    "0300_8007": "There was an unfinished print job when the printer lost power. If the model is still adhered to the build plate, you can try resuming the print job.",
    "0300_8008": "Printing Stopped because nozzle temperature problem.",
    "0300_800B": "The cutter is stuck. Please make sure the cutter handle is out.",
    "1200_8011": "AMS filament ran out. Please insert a new filament into the same AMS slot.",
    "0300_8011": "Detected build plate is not the same as the Gcode file. Please adjust slicer settings or use the correct plate.",
    "0701_8006": "Unable to feed filament into the extruder. This could be due to entangled filament or a stuck spool.",
    "0300_800A": "A Filament pile-up was detected by the AI Print Monitoring. Please clean the filament from the waste chute.",
    "07FF_8006": "Please feed filament into the PTFE tube until it can not be pushed any farther.",
    "07FF_C003": "Please pull out the filament on the spool holder. If this message persists, please check to see if there is filament broken in the extruder or PTFE Tube. (Connect PTFE tube if you are about to use AMS)",
    "07FF_C006": "Please feed filament into the PTFE tube until it can not be pushed any farther.",
    "12FF_8003": "Please pull out the filament on the spool holder. If this message persists, please check to see if there is filament broken in the extruder or PTFE Tube. (Connect PTFE tube if you are about to use AMS)",
    "12FF_C003": "Please pull out the filament on the spool holder. If this message persists, please check to see if there is filament broken in the extruder or PTFE Tube. (Connect PTFE tube if you are about to use AMS)",
    "12FF_8006": "Please feed filament into the PTFE tube until it can not be pushed any farther.",
    "12FF_C006": "Please feed filament into the PTFE tube until it can not be pushed any farther.",
    "12FF_8010": "Please check if the filament or the spool is stuck.",
    "07FF_8003": "Please pull out the filament on the spool holder. If this message persists, please check to see if there is filament broken in the extruder. (Connect PTFE tube if you are about to use AMS)",
    "1000_C003": "Enabling traditional timelapse might lead to defects. Please enable it as needed?",
    "1200_8004": "Failed to pull back the filament from the toolhead. Please check whether the filament is stuck. After troubleshooting, click the 'Retry' button.",
    "0C00_8005": "Purged filament has piled up in the waste chute, which may cause a tool head collision.",
    "0C00_C006": "Purged filament may have piled up in the waste chute.",
    "0500_4015": "There is not enough free storage space for the print job. Please format or clean MicroSD card to release available space.",
    "0500_C010": "MicroSD Card read/write exception. please reinsert or replace MicroSD Card .",
    "0300_8018": "Chamber temperature malfunction.",
    "0300_8019": "No build plate is placed.",
    "0300_8014": "The nozzle is covered with filaments, or the build plate is installed incorrectly. Please cancel this printing and clean the nozzle or adjust the build plate according to the actual status, or tap 'Resume' button to resume the print job.",
    "0300_8016": "The nozzle is clogged up with filaments. Please cancel this printing and clean the nozzle according to the actual status, or tap 'Resume' button to resume the print job.",
    "0300_4005": "The nozzle fan speed is abnormal.",
    "0300_400F": "No build plate is placed.",
    "0500_4002": "Unsupported print file path or name. Please resend the printing job.",
    "0500_8036": "Your sliced file is not consistent with the current printer model. Continue?",
    "0500_4037": "Your sliced file is not compatible with current printer model. This file can't be printed on this printer.",
    "0500_4038": "The nozzle diameter in sliced file is not consistent with the current nozzle setting. This file can't be printed.",
    "1000_C002": "Printing CF material with stainless steel may cause nozzle damage.",
    "1001_C001": "Timelapse is not supported because Spiral vase is enabled in slicing presets.",
    "1001_C002": "Timelapse is not supported because Print sequence is set to 'By object'.",
    "0501_4035": "The device is in the process of binding and cannot respond to new binding requests.",
    "0501_4032": "QR code binding is in progress, so device discovery binding cannot be performed. You can scan the QR code on the screen for binding or exit the QR code display page on screen and try device discovery binding.",
    "1202_8005": "Failed to feed the filament. Please load the filament, then click the 'Retry' button.",
    "1203_8005": "Failed to feed the filament. Please load the filament, then click the 'Retry' button.",
    "12FF_8005": "Failed to feed the filament. Please load the filament, then click the 'Retry' button.",
    "1202_8006": "Failed to feed the filament into the toolhead. Please check whether the filament is stuck. After troubleshooting, click the 'Retry' button.",
    "1203_8006": "Failed to feed the filament into the toolhead. Please check whether the filament is stuck. After troubleshooting, click the 'Retry' button.",
    "1202_8010": "Please check if the spool or filament is stuck. After troubleshooting, click the 'Retry' button.",
    "1203_8010": "Please check if the spool or filament is stuck. After troubleshooting, click the 'Retry' button.",
    "1202_8011": "AMS filament has run out. Please insert a new filament into the AMS and click the 'Retry' button.",
    "1203_8011": "AMS filament has run out. Please insert a new filament into the AMS and click the 'Retry' button.",
    "12FF_8011": "AMS filament has run out. Please insert a new filament into the AMS and click the 'Retry' button.",
    "1202_8012": "Failed to get AMS mapping table; please click the 'Retry' button to continue.",
    "1203_8012": "Failed to get AMS mapping table; please click the 'Retry' button to continue.",
    "12FF_8012": "Failed to get AMS mapping table; please click the 'Retry' button to continue.",
    "1202_8013": "Timeout while purging old filament. Please check if the filament is stuck or the extruder clogged. After troubleshooting, click the 'Retry' button.",
    "1203_8013": "Timeout while purging old filament. Please check if the filament is stuck or the extruder clogged. After troubleshooting, click the 'Retry' button.",
    "12FF_8013": "Timeout while purging old filament. Please check if the filament is stuck or the extruder clogged. After troubleshooting, click the 'Retry' button.",
    "1202_8015": "Failed to pull back the filament from the toolhead. Please check if the filament is stuck or is broken inside the extruder. After troubleshooting, click the 'Retry' button.",
    "1203_8015": "Failed to pull back the filament from the toolhead. Please check if the filament is stuck or is broken inside the extruder. After troubleshooting, click the 'Retry' button.",
    "1203_8001": "Failed to cut the filament. Please check the cutter. After troubleshooting, click the 'Retry' button.",
    "12FF_8001": "Failed to cut the filament. Please check the cutter. After troubleshooting, click the 'Retry' button.",
    "1203_8002": "The cutter is stuck. Please pull out the cutter handle and click the 'Retry' button.",
    "12FF_8002": "The cutter is stuck. Please pull out the cutter handle and click the 'Retry' button.",
    "1203_8003": "Failed to pull out the filament from the extruder. Please check whether the extruder is clogged or whether the filament is broken inside the extruder. After troubleshooting, click the 'Retry' button.",
    "1202_8004": "Failed to pull back the filament from the toolhead. Please check whether the filament is stuck. After troubleshooting, click the 'Retry' button.",
    "1203_8004": "Failed to pull back the filament from the toolhead. Please check whether the filament is stuck. After troubleshooting, click the 'Retry' button.",
    "12FF_8004": "Failed to pull back the filament from the toolhead. Please check whether the filament is stuck. After troubleshooting, click the 'Retry' button.",
    "0700_8001": "Failed to cut the filament. Please check the cutter. After troubleshooting, click the 'Retry' button.",
    "0701_8001": "Failed to cut the filament. Please check the cutter. After troubleshooting, click the 'Retry' button.",
    "0701_8002": "The cutter is stuck. Please pull out the cutter handle and click the 'Retry' button.",
    "0701_8007": "Failed to extrude the filament. Please check if the extruder clogged. After troubleshooting, click the 'Retry' button.",
    "0702_8001": "Failed to cut the filament. Please check the cutter. After troubleshooting, click the 'Retry' button.",
    "0702_8002": "The cutter is stuck. Please pull out the cutter handle and click the 'Retry' button.",
    "0702_8003": "Failed to pull out the filament from the extruder. Please check whether the extruder is clogged or whether the filament is broken inside the extruder. After troubleshooting, click the 'Retry' button.",
    "0702_8004": "Failed to pull back the filament from the toolhead to AMS. Please check whether the filament or the spool is stuck. After troubleshooting, click the 'Retry' button.",
    "0702_8005": "Failed to feed the filament outside the AMS. Please clip the end of the filament flat and check to see if the spool is stuck. After troubleshooting, click the 'Retry' button.",
    "0702_8006": "Failed to feed the filament into the toolhead. Please check whether the filament or the spool is stuck. After troubleshooting, click the 'Retry' button.",
    "0702_8007": "Failed to extrude the filament. Please check if the extruder clogged. After troubleshooting, click the 'Retry' button.",
    "0702_8010": "AMS assist motor is overloaded. Please check if the spool or filament is stuck. After troubleshooting, click the 'Retry' button.",
    "0703_8001": "Failed to cut the filament. Please check the cutter. After troubleshooting, click the 'Retry' button.",
    "0703_8002": "The cutter is stuck. Please pull out the cutter handle and click the 'Retry' button.",
    "0703_8003": "Failed to pull out the filament from the extruder. Please check whether the extruder is clogged or whether the filament is broken inside the extruder. After troubleshooting, click the 'Retry' button.",
    "0703_8004": "Failed to pull back the filament from the toolhead to AMS. Please check whether the filament or the spool is stuck. After troubleshooting, click the 'Retry' button.",
    "0703_8005": "Failed to feed the filament outside the AMS. Please clip the end of the filament flat and check to see if the spool is stuck. After troubleshooting, click the 'Retry' button.",
    "0703_8006": "Failed to feed the filament into the toolhead. Please check whether the filament or the spool is stuck. After troubleshooting, click the 'Retry' button.",
    "0703_8007": "Failed to extrude the filament. Please check if the extruder clogged. After troubleshooting, click the 'Retry' button.",
    "0703_8010": "AMS assist motor is overloaded. Please check if the spool or filament is stuck. After troubleshooting, click the 'Retry' button.",
    "0700_8013": "Timeout purging old filament: Please check if the filament is stuck or the extruder is clogged. After troubleshooting, click the 'Retry' button.",
    "0701_8013": "Timeout purging old filament: Please check if the filament is stuck or the extruder is clogged. After troubleshooting, click the 'Retry' button.",
    "0702_8013": "Timeout purging old filament: Please check if the filament is stuck or the extruder is clogged. After troubleshooting, click the 'Retry' button.",
    "0703_8013": "Timeout purging old filament: Please check if the filament is stuck or the extruder is clogged. After troubleshooting, click the 'Retry' button.",
    "07FF_8001": "Failed to cut the filament. Please check the cutter. After troubleshooting, click the 'Retry' button.",
    "07FF_8002": "The cutter is stuck. Please pull out the cutter handle and click the 'Retry' button.",
    "07FF_8005": "Failed to feed the filament outside the AMS. Please clip the end of the filament flat and check to see if the spool is stuck. After troubleshooting, click the 'Retry' button.",
    "07FF_8010": "AMS assist motor is overloaded. Please check if the spool or filament is stuck. After troubleshooting, click the 'Retry' button.",
    "07FF_8011": "AMS filament ran out. Please put a new filament into AMS and click the 'Retry' button.",
    "07FF_8013": "Timeout purging old filament: Please check if the filament is stuck or the extruder is clogged. After troubleshooting, click the 'Retry' button.",
    "1201_8001": "Failed to cut the filament. Please check the cutter. After troubleshooting, click the 'Retry' button.",
    "1202_8001": "Failed to cut the filament. Please check the cutter. After troubleshooting, click the 'Retry' button.",
    "1200_8002": "The cutter is stuck. Please pull out the cutter handle and click the 'Retry' button.",
    "1201_8002": "The cutter is stuck. Please pull out the cutter handle and click the 'Retry' button.",
    "1202_8002": "The cutter is stuck. Please pull out the cutter handle and click the 'Retry' button.",
    "1200_8003": "Failed to pull out the filament from the extruder. Please check whether the extruder is clogged or whether the filament is broken inside the extruder. After troubleshooting, click the 'Retry' button.",
    "1201_8003": "Failed to pull out the filament from the extruder. Please check whether the extruder is clogged or whether the filament is broken inside the extruder. After troubleshooting, click the 'Retry' button.",
    "1202_8003": "Failed to pull out the filament from the extruder. Please check whether the extruder is clogged or whether the filament is broken inside the extruder. After troubleshooting, click the 'Retry' button.",
    "1201_8004": "Failed to pull back the filament from the toolhead. Please check whether the filament is stuck. After troubleshooting, click the 'Retry' button.",
    "1200_8005": "Failed to feed the filament. Please load the filament, then click the 'Retry' button.",
    "1201_8005": "Failed to feed the filament. Please load the filament and then click the 'Retry' button.",
    "1201_8006": "Failed to feed the filament into the toolhead. Please check whether the filament is stuck. After troubleshooting, click the 'Retry' button.",
    "1201_8010": "Please check if the spool or filament is stuck. After troubleshooting, click the 'Retry' button.",
    "1201_8011": "AMS filament has run out. Please insert a new filament into the AMS and click the 'Retry' button.",
    "1200_8012": "Failed to get AMS mapping table. Please click the 'Retry' button to continue.",
    "1201_8012": "Failed to get AMS mapping table; please click the 'Retry' button to continue.",
    "1200_8013": "Timeout while purging old filament. Please check if the filament is stuck or the extruder clogged. After troubleshooting, click the 'Retry' button.",
    "1201_8013": "Timeout while purging old filament. Please check if the filament is stuck or the extruder clogged. After troubleshooting, click the 'Retry' button.",
    "1203_8014": "Failed to check the filament location in the tool head; please refer to the HMS. After troubleshooting, click the 'Retry' button.",
    "1201_8015": "Failed to pull back the filament from the toolhead. Please check if the filament is stuck or the filament is broken inside the extruder. After troubleshooting, click the 'Retry' button.",
    "0300_800D": "Some objects have fallen down, or the extruder is not extruding normally. If the defects are acceptable, click 'Resume' button to resume the print job.",
    "0C00_8001": "First layer defects were detected. If the defects are acceptable, click 'Resume' button to resume the print job.",
    "0500_8030": "",
    "0500_402E": "The system does not support the file system currently used by the Micro SD card. Please replace the Micro SD card or format the current Micro SD card to FAT32.",
    "0500_402F": "The Micro SD card sector data is damaged. Please use the SD card repair tool to repair or format it. If it still cannot be identified, please replace the Micro SD card.",
    "0501_4017": "Binding failed. Please retry or restart the printer and retry.",
    "0501_4018": "Binding configuration information parsing failed,  please try again.",
    "0501_4019": "The printer has already been bound. Please unbind it and try again.",
    "0501_401A": "Cloud access failed. Possible reasons include network instability caused by interference, inability to access the internet, or router firewall configuration restrictions. You can try moving the printer closer to the router or checking the router configuration and then try again.",
    "0501_401B": "Cloud response is invalid. If you have tried multiple times and are still failing, please contact customer service.",
    "0501_401C": "Cloud access is rejected. If you have tried multiple times and are still failing, please contact customer service.",
    "0501_401D": "Cloud access failed, which may be caused by network instability due to interference. You can try moving the printer closer to the router before you try again.",
    "0501_401E": "Cloud response is invalid. If you have tried multiple times and are still failing, please contact customer service.",
    "0501_401F": "Authorization timed out. Please make sure that your phone or PC has access to the internet, and ensure that the Bambu Studio/Bambu Handy APP is running in the foreground during the binding operation.",
    "0501_4020": "Cloud access rejected. If you have tried multiple times and are still failing, please contact customer service.",
    "0501_4021": "Cloud access failed, which may be caused by network instability due to interference. You can try moving the printer closer to the router before you try again.",
    "0501_4022": "Cloud response is invalid. If you have tried multiple times and are still failing, please contact customer service.",
    "0501_4023": "Cloud access rejected. If you have tried multiple times and are still failing, please contact customer service.",
    "0501_4024": "Cloud access failed. Possible reasons include network instability caused by interference, inability to access the internet, or router firewall configuration restrictions. You can try moving the printer closer to the router or checking the router configuration before you try again.",
    "0501_4025": "Cloud response is invalid. If you have tried multiple times and are still failing, please contact customer service.",
    "0501_4026": "Cloud access rejected. If you have tried multiple times and are still failing, please contact customer service.",
    "0501_4027": "Cloud access failed; this may be caused by network instability due to interference. You can try moving the printer closer to the router before you try again.",
    "0501_4028": "Cloud response is invalid. If you have tried multiple times and are still failing, please contact customer service.",
    "0501_4029": "Cloud access is rejected. If you have tried multiple times and are still failing, please contact customer service.",
    "0501_4031": "Device discovery binding is in progress, and the QR code cannot be displayed on the screen. You can wait for the binding to finish or abort the device discovery binding process in the APP/Studio and retry scanning the QR code on the screen for binding.",
    "0501_4034": "The slicing progress has not been updated for a long time, and the printing task has exited. Please confirm the parameters and reinitiate printing.",
    "07FF_8004": "Failed to pull back the filament from the toolhead to AMS. Please check whether the filament or the spool is stuck. After troubleshooting, click the 'retry' button.",
    "0C00_800A": "The detected build plate is not the same as in G-code.",
    "12FF_8007": "Check nozzle. Click 'Done' if filament was extruded, otherwise push filament forward slightly and click 'Retry.'",
    "0300_8012": "",
    "0500_402D": "System exception.",
    "0500_4027": "Cloud access failed; this may be caused by network instability due to interference. You can try moving the printer closer to the router before you try again.",
    "0500_4028": "Cloud response is invalid. If you have tried multiple times and are still failing, please contact customer service.",
    "0500_4029": "Cloud access is rejected. If you have tried multiple times and are still failing, please contact customer service.",
    "0500_402A": "Failed to connect to the router, which may be caused by wireless interference or being too far away from the router. Please try again or move the printer closer to the router and try again.",
    "0500_402C": "Failed to obtain IP address, which may be caused by wireless interference resulting in data transmission failure or DHCP address pool of the router being full. Please move the printer closer to the router and try again. If the issue persists, please check router settings to see whether the IP addresses have been exhausted.",
    "0500_4016": "The MicroSD Card is write-protected. Please replace the MicroSD Card.",
    "0500_401A": "Cloud access failed. Possible reasons include network instability caused by interference, inability to access the internet, or router firewall configuration restrictions. You can try moving the printer closer to the router or checking the router configuration and then try again.",
    "0500_401B": "Cloud response is invalid. If you have tried multiple times and are still failing, please contact customer service.",
    "0500_401C": "Cloud access is rejected. If you have tried multiple times and are still failing, please contact customer service.",
    "0500_401D": "Cloud access failed, which may be caused by network instability due to interference. You can try moving the printer closer to the router before you try again.",
    "0500_401E": "Cloud response is invalid. If you have tried multiple times and are still failing, please contact customer service.",
    "0500_4020": "Cloud access rejected. If you have tried multiple times and are still failing, please contact customer service.",
    "0500_4021": "Cloud access failed, which may be caused by network instability due to interference. You can try moving the printer closer to the router before you try again.",
    "0500_4022": "Cloud response is invalid. If you have tried multiple times and are still failing, please contact customer service.",
    "0500_4023": "Cloud access rejected. If you have tried multiple times and are still failing, please contact customer service.",
    "0500_4024": "Cloud access failed. Possible reasons include network instability caused by interference, inability to access the internet, or router firewall configuration restrictions. You can try moving the printer closer to the router or checking the router configuration before you try again.",
    "0500_4025": "Cloud response is invalid. If you have tried multiple times and are still failing, please contact customer service.",
    "0500_4026": "Cloud access rejected. If you have tried multiple times and are still failing, please contact customer service.",
    "0500_401F": "Authorization timed out. Please make sure that your phone or PC has access to the internet, and ensure that the Bambu Studio/Bambu Handy APP is running in the foreground during the binding operation.",
    "0500_402B": "Router connection failed due to incorrect password. Please check the password and try again.",
    "0500_4017": "Binding failed. Please retry or restart the printer and retry.",
    "0500_4018": "Binding configuration information parsing failed,  please try again.",
    "0500_4019": "The printer has already been bound. Please unbind it and try again.",
    "07FF_8007": "Please observe the nozzle. If the filament has been extruded, click 'Done'; if it is not, please push the filament forward slightly and then click 'Retry'.",
    "1201_4001": "Filament is still loaded from the AMS when it has been disabled. Please unload AMS filament, load from spool holder, and restart print job.",
    "1202_4001": "Filament is still loaded from the AMS when it has been disabled. Please unload AMS filament, load from spool holder, and restart print job.",
    "1203_4001": "Filament is still loaded from the AMS when it has been disabled. Please unload AMS filament, load from spool holder, and restart print job.",
    "12FF_4001": "Filament is still loaded from the AMS when it has been disabled. Please unload AMS filament, load from spool holder, and restart print job.",
    "1200_4001": "Filament is still loaded from the AMS when it has been disabled. Please unload AMS filament, load from spool holder, and restart print job.",
    "0300_800E": "The print file is not available. Please check to see if the storage media has been removed.",
    "0500_8013": "The print file is not available. Please check to see if the storage media has been removed.",
    "0500_4012": "The door seems to be open, so printing was paused.",
    "0300_800F": "The door seems to be open, so printing was paused.",
    "0500_C011": "",
    "0500_400C": "Please insert a MicroSD card and restart the printing job.",
    "07FF_8012": "Failed to get AMS mapping table; please click 'Retry' to continue.",
    "07FF_4001": "Filament is still loaded from the AMS after it has been disabled. Please unload the filament, load from the spool holder, and restart printing.",
    "0300_800C": "Skipping step detected, auto-recover complete; please resume print and check if there are any layer shift problems.",
    "0500_400E": "Printing was cancelled.",
    "0C00_8009": "Build plate localization marker was not found.",
    "0500_400D": "Please run a self-test and restart the printing job.",
    "0500_400B": "There was a problem downloading a file. Please check you network connection and resend the printing job.",
    "0300_400A": "Mechanical resonance frequency identification failed.",
    "0300_8009": "Heatbed temperature malfunction.",
    "0300_400E": "The motor self-check failed.",
    "0700_8012": "Failed to get AMS mapping table; please click 'Retry' to continue.",
    "0701_8012": "Failed to get AMS mapping table; please click 'Retry' to continue.",
    "0702_8012": "Failed to get AMS mapping table; please click 'Retry' to continue.",
    "0703_8012": "Failed to get AMS mapping table; please click 'Retry' to continue.",
    "0701_4001": "Filament is still loaded from the AMS after it has been disabled. Please unload the filament, load from the spool holder, and restart printing.",
    "0702_4001": "Filament is still loaded from the AMS after it has been disabled. Please unload the filament, load from the spool holder, and restart printing.",
    "0703_4001": "Filament is still loaded from the AMS after it has been disabled. Please unload the filament, load from the spool holder, and restart printing.",
    "0C00_8002": "Spaghetti failure was detected.",
    "0C00_C003": "Possible defects were detected in the first layer.",
    "0C00_C004": "Possible spaghetti failure was detected.",
    "0500_4007": "Print jobs are not allowed to be sent while force updating or when repair updating is required.",
    "0500_4008": "Starting printing failed. please power cycle the printer and resend the print job.",
    "0500_4009": "Print jobs are not allowed to be sent while updating logs.",
    "0500_400A": "The file name is not supported. Please rename and restart the printing job.",
    "0300_8001": "Printing was paused by the user. You can tap 'Resume' to resume the print job.",
    "0500_4001": "Failed to connect to Bambu Cloud. Please check your network connection.",
    "0500_4005": "Print jobs are not allowed to be sent while updating firmware.",
    "0500_4006": "There is not enough free storage space for the print job. Restoring to factory settings can release available space.",
    "0300_4001": "The printer timed out waiting for the nozzle to cool down before homing.",
    "0300_4006": "The nozzle is clogged.",
    "0300_4008": "The AMS failed to change filament.",
    "0300_4009": "Homing XY axis failed.",
    "0300_400B": "Internal communication exception.",
    "0300_400C": "Printing was cancelled.",
    "0300_400D": "Resume failed after power loss.",
    "0300_8000": "Printing was paused for unknown reason. You can tap 'Resume' to resume the print job.",
    "0300_4003": "Nozzle temperature malfunction.",
    "0300_4004": "Heatbed temperature malfunction.",
}
# UNIQUE_ID=Y329g6Nq

HMS_SEVERITY_LEVELS = {
    "default": "unknown",
    1: "fatal",
    2: "serious",
    3: "common",
    4: "info"
}

HMS_MODULES = {
    "default": "unknown",
    0x05: "mainboard",
    0x0C: "xcam",
    0x07: "ams",
    0x08: "toolhead",
    0x03: "mc"
}

class SdcardState(Enum):
    NO_SDCARD                           = 0x00000000,
    HAS_SDCARD_NORMAL                   = 0x00000100,
    HAS_SDCARD_ABNORMAL                 = 0x00000200,
    SDCARD_STATE_NUM                    = 0x00000300,

class Home_Flag_Values(IntEnum):
    X_AXIS                              = 0x00000001,
    Y_AXIS                              = 0x00000002,
    Z_AXIS                              = 0x00000004,
    VOLTAGE220                          = 0x00000008,
    XCAM_AUTO_RECOVERY_STEP_LOSS        = 0x00000010,
    CAMERA_RECORDING                    = 0x00000020,
    # Gap
    AMS_CALIBRATE_REMAINING             = 0x00000080,
    SD_CARD_PRESENT                     = 0x00000100,
    SD_CARD_ABNORMAL                    = 0x00000200,
    AMS_AUTO_SWITCH                     = 0x00000400,
    # Gap
    XCAM_ALLOW_PROMPT_SOUND             = 0x00020000,
    WIRED_NETWORK                       = 0x00040000,
    FILAMENT_TANGLE_DETECT_SUPPORTED    = 0x00080000,
    FILAMENT_TANGLE_DETECTED            = 0x00100000,
    SUPPORTS_MOTOR_CALIBRATION          = 0x00200000,
    # Gap
    DOOR_OPEN                           = 0x00800000,
    # Gap
    INSTALLED_PLUS                      = 0x04000000,
    SUPPORTED_PLUS                      = 0x08000000,
    # Gap

class BambuUrl(Enum):
    LOGIN = 1,
    TFA_LOGIN = 2,
    EMAIL_CODE = 3,
    BIND = 4,
    SLICER_SETTINGS = 5,
    TASKS = 6,

BAMBU_URL = {
    BambuUrl.LOGIN: 'https://api.bambulab.com/v1/user-service/user/login',
    BambuUrl.TFA_LOGIN: 'https://bambulab.com/api/sign-in/tfa',
    BambuUrl.EMAIL_CODE: 'https://api.bambulab.com/v1/user-service/user/sendemail/code',
    BambuUrl.BIND: 'https://api.bambulab.com/v1/iot-service/api/user/bind',
    BambuUrl.SLICER_SETTINGS: 'https://api.bambulab.com/v1/iot-service/api/slicer/setting?version=undefined',
    BambuUrl.TASKS: 'https://api.bambulab.com/v1/user-service/my/tasks',
}
